///////////////////////////////////////////////////////////////////////////////////////////////////
//         glibMemPool.h
///////////////////////////////////////////////////////////////////////////////////////////////////

/*! \class        glibMemPool.h
*  \brief        <brief description> 
*                <long description> 
*  \version      1.0 
*  \date         20.01.2005
*  \author       responsible: [kt]
*/



#ifndef _GLIBMEMPOOL_H_HEADER_INCLUDED_
#define _GLIBMEMPOOL_H_HEADER_INCLUDED_

template<class _T,
		 int nPoolSize = 1024,
		 int nSlotsIncrement = 1>

		 class glibMemPool
{

public:

	glibMemPool()
	{
		m_pPoolSlots = NULL;
		
		m_nSlotCount	 = 0;
		m_nSlotCapacity	 = 0;
	}

	~glibMemPool()
	{
		if (m_pPoolSlots != NULL) {
			
			// Delete all nodepools
			for (int nCount = 0; nCount < m_nSlotCapacity; nCount++) {
				
				// If nodepool exists in slot ...
				if (m_pPoolSlots[nCount] != NULL) {
					
					//delete/*((void*)&g_oHeapAllocator, false)*/[] m_pNodePoolSlots[nCount];
					delete m_pPoolSlots[nCount];
				}
			}
			
			// Delete nodepool container
			delete m_pPoolSlots;
		}
	}

	_T*	New()
	{
		// Try to find a slot with free antries
		int nIndex = FindSlot();
		
		// if not found .....
		if (nIndex < 0) {
			// Create new one
			nIndex = NewSlot();
			assert(nIndex >= 0);
		}
		
		// Return new node from slot
		return m_pPoolSlots[nIndex]->New();
	}

	_T*	New(const _T* pOther)
	{
		// Try to find a slot with free antries
		int nIndex = FindSlot();
		
		// if not found .....
		if (nIndex < 0) {
			// Create new one
			nIndex = NewSlot();
			assert(nIndex >= 0);
		}
		
		// Return new node from slot
		return m_pPoolSlots[nIndex]->New(pOther);
	}

	void Delete(_T *pPtr)
	{
		// find the slot which holds the specified node ...
		int nIndex = FindSlotForAddress(pPtr);
		
		// If no slot is found for specified address, this node was't created within that node
		assert(nIndex >= 0);
		
		// Delete this node from slot
		m_pPoolSlots[nIndex]->Delete(pPtr);
	}
	
	void operator=(const glibMemPool& oOther) {

		m_nSlotCapacity = oOther.m_nSlotCapacity;
		m_nSlotCount = m_nSlotCount;

		if (oOther.m_pPoolSlots != NULL) {

			m_pPoolSlots = m_pPoolSlots = new tPoolSlotType*[m_nSlotCapacity];

			for (nCount = 0; nCount < m_nSlotCapacity; nCount++) {
				
				if (oOther.m_pPoolSlots[nCount] != NULL) {		
					// Copy the slot with all it's contents
					*(m_pPoolSlots[nCount]) = *(oOther.m_pPoolSlots[nCount]);
				}
				else {
					// Otherwise simply set the slow = NULL
					m_pPoolSlots[nCount] = NULL;
				}
			}


		}
		else {
			m_pPoolSlots = NULL;
		}
		

	}

private:

	typedef glibMemPoolSlot<_T, nPoolSize> tPoolSlotType ;

	int FindSlot()
	{
		if (m_pPoolSlots == NULL)
			return -1;
		
		for (int nCount = 0; nCount < m_nSlotCapacity; nCount++) {
			
			if (m_pPoolSlots[nCount] != NULL)
				if (m_pPoolSlots[nCount]->HasFreeNodes())
					return nCount;
				
		}
		return -1;
	}
	
	int NewSlot()
	{
		// if container is not existant create it
		if (m_pPoolSlots == NULL) {
			
			m_pPoolSlots = new tPoolSlotType*[nSlotsIncrement];
			
			m_nSlotCapacity = nSlotsIncrement;
			for (int nCount = 0; nCount < m_nSlotCapacity; nCount++) {
				m_pPoolSlots[nCount] = NULL;
			}
		}
		
		// Search for free place to attach new slot
		int nCount;
		for (nCount = 0; nCount < m_nSlotCapacity; nCount++) {
			
			if (m_pPoolSlots[nCount] == NULL) {
				
				m_pPoolSlots[nCount] = new tPoolSlotType;
				
				m_nSlotCount++;
				return nCount;
			}
		}
		
		// Reorganize slots !
		
		tPoolSlotType** pTemp;
		
		pTemp = new tPoolSlotType*[m_nSlotCapacity + nSlotsIncrement];
		
		for (nCount = 0; nCount < m_nSlotCapacity + nSlotsIncrement; nCount++) {
			pTemp[nCount] = NULL;
		}
		
		
		for (nCount = 0; nCount < m_nSlotCapacity; nCount++) {
			pTemp[nCount] = m_pPoolSlots[nCount];  
		}
		
		m_nSlotCapacity += nSlotsIncrement;
		
		
		delete m_pPoolSlots;
		
		m_pPoolSlots = pTemp;
		
		// try again
		for (nCount = 0; nCount < m_nSlotCapacity; nCount++) {
			
			if (m_pPoolSlots[nCount] == NULL) {
				
				m_pPoolSlots[nCount] = GLIB_NEW_BASIC tPoolSlotType;
				
				m_nSlotCount++;
				return nCount;
			}
		}
		
		// This may not happen
		GLIB_BASIC_ASSERT(false);
		
		return -1;
	}
	
	int FindSlotForAddress(_T* pNode)
	{
		if (m_pPoolSlots == NULL)
			return -1;
		
		for (int nCount = 0; nCount < m_nSlotCapacity; nCount++) {
			
			if (m_pPoolSlots[nCount] != NULL)
				if (m_pPoolSlots[nCount]->IsNodeInSlot(pNode))
					return nCount;
				
		}
		return -1;
	};

	glibMemPoolSlot<_T, nPoolSize>**							m_pPoolSlots;
	
	int															m_nSlotCount;
	int															m_nSlotCapacity;

};

#endif //_GLIBMEMPOOL_H_HEADER_INCLUDED_

