///////////////////////////////////////////////////////////////////////////////////////////////////
//         glibMemPoolSlot.h
///////////////////////////////////////////////////////////////////////////////////////////////////

/*! \class        glibMemPoolSlot.h
*  \brief        <brief description> 
*                <long description> 
*  \version      1.0 
*  \date         20.01.2005
*  \author       responsible: [kt]
*/



#ifndef _GLIBMEMPOOLSLOT_H_HEADER_INCLUDED_
#define _GLIBMEMPOOLSLOT_H_HEADER_INCLUDED_

template<class _T,
		 int nPoolSize> 
		 class glibMemPoolSlot 
{
public:

	glibMemPoolSlot()
	{
		m_nNodePoolSize			= 0;
		m_nUsedNodes			= 0;
		m_nNodePoolCapacity		= 0;
		m_nUsageMapCapacity		= 0;
		
		m_nFreePointer			= -1;
		
		m_pNodepool				= NULL;
		m_pUsageMap				= NULL;
		
		
		InitializePool(nPoolSize);
	};

	~glibMemPoolSlot()
	{
		DeletePool();
	}
	
	
	bool IsNodeInSlot(_T* pNode)
	{
		unsigned int unAddress = (unsigned int)pNode;
		unsigned int unBaseAddress = (unsigned int)m_pNodepool;
		unsigned int unTopAddress = unBaseAddress + 
									m_nNodePoolCapacity * sizeof(_T);
		
		if ((unAddress >= unBaseAddress) &&
			(unAddress < unTopAddress ))
			return true;
		
		return false;
	}

	bool HasFreeNodes()
	{
		if (m_nUsedNodes >= m_nNodePoolCapacity) {
			
			return false;
		}
		return true;
	}
	
	_T* New()
	{
		int nFreeIndex = FindFreeEntry();
		
		if (nFreeIndex < 0)
			return NULL;
		
		ReserveEntry(nFreeIndex);
		
		new(&(m_pNodepool[nFreeIndex]), false) _T;
		//GLIB_MEM_BASIC_CONSTRUCT(&(m_pNodepool[nFreeIndex]), _T );
		
		return &(m_pNodepool[nFreeIndex]);
	}

	_T* New(const _T* pOther)
	{
		int nFreeIndex = FindFreeEntry();
		
		if (nFreeIndex < 0)
			return NULL;
		
		ReserveEntry(nFreeIndex);
		
		GLIB_MEM_BASIC_COPY_CONSTRUCT(&(m_pNodepool[nFreeIndex]), _T, *pOther);
		
		return &(m_pNodepool[nFreeIndex]);
	}

	void Delete(_T* pNode)
	{
		unsigned int unNodeAddress = (unsigned int)pNode;
		unsigned int unBaseAddress = (unsigned int)m_pNodepool;
		
		GLIB_BASIC_ASSERT(unNodeAddress >= unBaseAddress);
		
		int nIndex = (unNodeAddress - unBaseAddress) / sizeof(_T);
		
		//GLIB_MEM_BASIC_DESTRUCT(, _T);
		//GLIB_MEM_DELETE_OP(&(m_pNodepool[nIndex]));

		// Call destrutor of object
		::operator delete(&(m_pNodepool[nIndex]), &(m_pNodepool[nIndex]), true);

		FreeEntry(nIndex);
	}

	void operator=(const glibMemPoolSlot& oOther) {

		// Cleanup pool before copy
		DeletePool();

		m_nNodePoolSize =		oOther.m_nNodePoolSize;
		m_nUsedNodes =			oOther.m_nUsedNodes;
		m_nNodePoolCapacity =	oOther.m_nNodePoolCapacity;
		m_nUsageMapCapacity =	oOther.m_nUsageMapCapacity;

		m_nFreePointer =		oOther.m_nFreePointer;
		
		if (oOther.m_pNodepool != NULL) {

			m_pNodepool = (_T*)GLIB_MEM_BASIC_MALLOC(sizeof(_T) * nPoolSize);
		
			memcpy(m_pNodepool, oOther.m_pNodepool, sizeof(_T) * nPoolSize);

		}
		else {
			m_pNodepool = NULL;
		}

		if (oOther.m_pUsageMap != NULL) {

			m_nUsageMapCapacity = GetUsageMapSize(nPoolsize);
			
			m_pUsageMap = GLIB_NEW_BASIC unsigned int [m_nUsageMapCapacity];

			memcpy(m_pUsageMap, oOther.m_pUsageMap, sizeof(unsigned int) * m_nUsageMapCapacity);

		}
		else {

			m_pUsageMap = NULL;

		}
	}

private:

	void InitializePool(int nPoolsize)
	{
		GLIB_BASIC_ASSERT(m_pNodepool == NULL);
		GLIB_BASIC_ASSERT(m_pUsageMap == NULL);
		
		
		m_pNodepool = (_T*)GLIB_MEM_BASIC_MALLOC(sizeof(_T) * nPoolsize);
		
		m_nUsageMapCapacity = GetUsageMapSize(nPoolsize);
		
		m_pUsageMap = GLIB_NEW_BASIC unsigned int [m_nUsageMapCapacity];
		
		m_nNodePoolSize = 0;
		m_nUsedNodes = 0;
		m_nNodePoolCapacity = nPoolsize;
		m_nFreePointer = -1;		// Points to the first free entry
		
		// Clear usage map
		memset(m_pUsageMap, 0, m_nUsageMapCapacity * sizeof(unsigned int));

	}
	
	void DeletePool()
	{
		if (m_pNodepool != NULL) {
			GLIB_MEM_BASIC_FREE(m_pNodepool);
			m_pNodepool = NULL;
		}
		
		if (m_pUsageMap != NULL) {
			GLIB_DELETE_BASIC(m_pUsageMap);
			m_pUsageMap = NULL;
		}
		
		m_nUsageMapCapacity = 0;
		m_nNodePoolSize = 0;
		m_nUsedNodes = 0;
		m_nNodePoolCapacity = 0;
		m_nFreePointer = 0;	
	}

	int FindFreeEntry() 
	{
		
		if (m_nNodePoolSize < m_nNodePoolCapacity) {
			
			// We can lineary increase node pool size until we have reached the top of capacity
			return m_nNodePoolSize++;
		}
		
		// If capacity is exceeded, return -1 to signal that no new nodes are available
		if (m_nUsedNodes >= m_nNodePoolCapacity) {
			
			m_nFreePointer = -1;
			return -1;
		}
		
		
		int nCounterStart = 0;
		
		// Usig the knowledge about the first free entry to optimize performance a bit
		if (m_nFreePointer >= 0) {
			
			nCounterStart = m_nFreePointer;
		}
		
		// When having reached the top of capacity, we can search the whole 
		for (int nCount = m_nFreePointer; nCount < m_nUsageMapCapacity; nCount++) {
			
			// We can skip entries where all bits are set !! These are full and contain no free entries.
			if (m_pUsageMap[nCount] != 0xffffffff) {
				
				int nEntryIndexOffset = nCount << 5;
				
				unsigned int nRotator = 1;
				int nOffs = 0;
				
				for (; nRotator != 0; nRotator <<= 1, nOffs++) {
					
					if ((m_pUsageMap[nCount] & nRotator ) == 0) {
						
						int nInx = nEntryIndexOffset + nOffs;
						
						if (nInx < m_nNodePoolCapacity) {
							
							m_nFreePointer = nCount;
							return nInx;
						}
						
						
						m_nFreePointer = -1;
						// No entry left !! Pool is full
						return -1;
						
					}
				}
			}
		}
		// No entry left !! Pool is full
		return -1;
		
	}

	bool ReserveEntry(int nEntry) 
	{
		GLIB_BASIC_ASSERT(nEntry >= 0);
		
		int nOffs = nEntry & 0x0000001f;
		int nMapEntry = nEntry >> 5;
		unsigned int unMask = (((unsigned int)1) << nOffs);
		
		// Entry must be free to be reserved !!
		GLIB_BASIC_ASSERT(((m_pUsageMap[nMapEntry] & (unMask)) == 0));
		
		m_pUsageMap[nMapEntry] |= unMask;
		
		m_nUsedNodes++;
		
		
		return true;
	}

	bool FreeEntry(int nEntry)
	{
		GLIB_BASIC_ASSERT(nEntry >= 0);
		
		int nOffs = nEntry & 0x0000001f;
		int nMapEntry = nEntry >> 5;
		unsigned int unMask = (((unsigned int)1) << nOffs);
		
		// Entry must be free to be reserved !!
		GLIB_BASIC_ASSERT((m_pUsageMap[nMapEntry] & (unMask)) == unMask);
		
		
		m_pUsageMap[nMapEntry] &= ~unMask;
		
		m_nUsedNodes--;
		
		if ((m_nFreePointer > nMapEntry) || (m_nFreePointer < 0))
			m_nFreePointer = nMapEntry;
		
		return true;
	}



	int GetUsageMapSize(int nPoolsize)
	{
		return (nPoolsize >> 5) + 1;
	}


	int						m_nNodePoolSize;
	int						m_nUsedNodes;
	int						m_nNodePoolCapacity;
	int						m_nUsageMapCapacity;
	
	int						m_nFreePointer;
	
	_T*						m_pNodepool;
	unsigned int*			m_pUsageMap;

};


#endif //_GLIBMEMPOOLSLOT_H_HEADER_INCLUDED_

