///////////////////////////////////////////////////////////////////////////////////////////////////
//         glibUtil_AVLTree.h
///////////////////////////////////////////////////////////////////////////////////////////////////

/*! \class        glibUtil_AVLTree.h
*  \brief        <brief description> 
*                <long description> 
*  \version      1.0 
*  \date         20.01.2005
*  \author       responsible: [kt]
*/



#ifndef _GLIBUTIL_AVLTREE_H_HEADER_INCLUDED_
#define _GLIBUTIL_AVLTREE_H_HEADER_INCLUDED_

#define ___TREE_TYPE___ glibUtilAVLTree <_Key, _Value, nNodePoolSize, nSlotsIncrement, _Comperator> 
#define ___NODE_TYPE___ glibUtilAVLNode<_Key, _Value, _Comperator>
#define ___ITERATOR_TYPE___ glibUtilAVLNode<_Key, _Value, _Comperator>::iterator
#define ___CONST_ITERATOR_TYPE___ glibUtilAVLNode<_Key, _Value, _Comperator>::const_iterator

template <class _Key, 
		  class _Value, 
		  int nNodePoolSize = 1024,
		  int nSlotsIncrement = 16,
		  class _Comperator = glibAvlNode_StdComperator<_Key> > 
class glibUtilAVLTree :
	glibUtilAVLTree_NodeCreator<_Key, _Value, _Comperator>
{
public:

	typedef ___NODE_TYPE___ node_type;
	typedef typename ___ITERATOR_TYPE___ iterator_type;
	typedef typename ___CONST_ITERATOR_TYPE___ const_iterator_type;

	glibUtilAVLTree()
	{
		m_oDummyRootNode.SetDummyRoot();
		m_bIterating	 = false;
	}

	~glibUtilAVLTree()
	{
		Clear();

	}

	void operator=(const ___TREE_TYPE___& oOther) {

		// May not copy while iterating
		assert(!m_bIterating);

		m_oDummyRootNode.Clone(oOther.m_oDummyRootNode, *this);

	}

	void Clear()
	{
		m_oDummyRootNode.DeleteChildren(m_oNodePool);
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Add and remove by using pool
	//-----------------------------------------------------------------------------------------------------------------

	bool AddNode(___NODE_TYPE___& oNewNode)
	{

		// And add it to the tree
		return m_oDummyRootNode.AddKey(&oNewNode, bStrict);
	}

	void RemoveNode(___NODE_TYPE___& oNode)
	{
		
		oNode.RemoveThis();
			
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Add and remove external nodes (will neither delere, nor create)
	//-----------------------------------------------------------------------------------------------------------------

	bool Add(const _Key& oKey, const _Value& oValue, bool bStrict = true)
	{

		___NODE_TYPE___* pNewNode = m_oNodePool.New();
		
		pNewNode->SetKeyValue(oKey, oValue);

		bool bSucc = m_oDummyRootNode.AddNode(*pNewNode, bStrict);

		if (!bSucc) {
			m_oNodePool.Delete(pNewNode);
		}
	
		return bSucc;

	}

	void Remove(const _Key& oKey)
	{
		___NODE_TYPE___* pNode = m_oDummyRootNode.FindKey(oKey);
		
		// Remove me from tree
		if (pNode != NULL) {
			pNode->RemoveThis();
			m_oNodePool.Delete(pNode);
		}
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Search functions
	//-----------------------------------------------------------------------------------------------------------------	
	
	___NODE_TYPE___* FindKey(const _Key& oKey)
	{
		return m_oDummyRootNode.FindKey(oKey);
	}

	___NODE_TYPE___* FindKeyMin(const _Key& oKey)
	{
		return m_oDummyRootNode.FindKeyMin(oKey);
	}

	___NODE_TYPE___* FindKeyMax(const _Key& oKey)
	{
		return m_oDummyRootNode.FindKeyMax(oKey);
	}

	___NODE_TYPE___* FindSmallest()
	{
		return m_oDummyRootNode.FindSmallest();
	}
	
	___NODE_TYPE___* FindBiggest()
	{
		return m_oDummyRootNode.FindBiggest();
	}

	const ___NODE_TYPE___* FindKey(const _Key& oKey) const
	{
		return m_oDummyRootNode.FindKey(oKey);
	}

	const ___NODE_TYPE___* FindKeyMin(const _Key& oKey) const
	{
		return m_oDummyRootNode.FindKeyMin(oKey);
	}

	const ___NODE_TYPE___* FindKeyMax(const _Key& oKey) const
	{
		return m_oDummyRootNode.FindKeyMax(oKey);
	}

	const ___NODE_TYPE___* FindSmallest() const
	{
		return m_oDummyRootNode.FindSmallest();
	}
	
	const ___NODE_TYPE___* FindBiggest() const
	{
		return m_oDummyRootNode.FindBiggest();
	}
	//-----------------------------------------------------------------------------------------------------------------
	//	Iterator access
	//-----------------------------------------------------------------------------------------------------------------		

	typename ___ITERATOR_TYPE___ Find(const _Key& oKey)
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindKey(oKey));
	}

	typename ___CONST_ITERATOR_TYPE___ Find(const _Key& oKey) const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindKey(oKey));
	}

	typename ___ITERATOR_TYPE___ FindMin(const _Key& oKey)
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMin(oKey));
	}

	typename ___CONST_ITERATOR_TYPE___ FindMin(const _Key& oKey) const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMin(oKey));
	}

	typename ___ITERATOR_TYPE___ FindMax(const _Key& oKey)
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMax(oKey));
	}

	typename ___CONST_ITERATOR_TYPE___ FindMax(const _Key& oKey) const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMax(oKey));
	}

	typename ___ITERATOR_TYPE___ FindFirst()
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindSmallest());
	}

	typename ___CONST_ITERATOR_TYPE___ FindFirst() const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindSmallest());
	}

	typename ___ITERATOR_TYPE___ FindLast()
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindBiggest());
	}

	typename ___CONST_ITERATOR_TYPE___ FindLast() const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindBiggest());
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Enumerating/iterating tree
	//-----------------------------------------------------------------------------------------------------------------		

	void BeginEnumerateNodes()
	{
		assert(!m_bEnumerating);
		m_oIt = m_oDummyRootNode.FindSmallest();

		m_bEnumerating = true;
	}

	bool IsEnd()
	{
		assert(m_bEnumerating);
		return m_oIt.IsValid();
	}

	___NODE_TYPE___* GetNextNode()
	{
		assert(m_bEnumerating);
		return m_oIt++;
	}

	void EndEnumerateNodes()
	{
		assert(m_bEnumerating);
		m_oIt.Clear();

		m_bEnumerating = false;
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Direct tree access
	//-----------------------------------------------------------------------------------------------------------------	

	___NODE_TYPE___* operator->()
	{
		return &m_oDummyRootNode;
	}

	const ___NODE_TYPE___* operator->() const
	{
		return &m_oDummyRootNode;
	}

	virtual ___NODE_TYPE___* CreateNode() {

		return m_oNodePool.New();
	}

	virtual void DeleteNode(___NODE_TYPE___* pNode) {

		return m_oNodePool.Delete(pNode);
	}

private:

	typename ___NODE_TYPE___								m_oDummyRootNode;
	typename ___ITERATOR_TYPE___							m_oIt;

	glibMemPool<node_type, nNodePoolSize, nSlotsIncrement>	m_oNodePool;

	bool													m_bIterating;

};

#undef ___TREE_TYPE___
#define ___TREE_TYPE___ glibUtilAVLTree_StaticPool <_Key, _Value, _Comperator> 

template <class _Key, 
		  class _Value, 
		  class _Comperator = glibAvlNode_StdComperator<_Key> > 
class glibUtilAVLTree_StaticPool :
	glibUtilAVLTree_NodeCreator<_Key, _Value, _Comperator>
{
public:

	typedef ___NODE_TYPE___ node_type;
	typedef typename ___ITERATOR_TYPE___ iterator_type;
	typedef typename ___CONST_ITERATOR_TYPE___ const_iterator_type;
	typedef typename glibMemPool<node_type, 1024, 16> pool_type;

	glibUtilAVLTree_StaticPool()
	{
		m_oDummyRootNode.SetDummyRoot();
		m_bIterating	 = false;
	}

	~glibUtilAVLTree_StaticPool()
	{
		// Destroy the tree recoursively (this is nessecary for the static tree)
		m_oDummyRootNode.DeleteChildren(*this);
	}

	void operator=(const ___TREE_TYPE___& oOther) {

		// May not copy while iterating
		assert(!m_bIterating);

		m_oDummyRootNode.Clone(oOther.m_oDummyRootNode, *this);

	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Add and remove by using pool
	//-----------------------------------------------------------------------------------------------------------------

	bool AddNode(___NODE_TYPE___& oNewNode)
	{

		// And add it to the tree
		return m_oDummyRootNode.AddKey(&oNewNode, bStrict);
	}

	void RemoveNode(___NODE_TYPE___& oNode)
	{
		
		oNode.RemoveThis();
			
	}

	void Clear()
	{
		m_oDummyRootNode.DeleteChildren(*this);
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Add and remove external nodes (will neither delere, nor create)
	//-----------------------------------------------------------------------------------------------------------------

	bool Add(const _Key& oKey, const _Value& oValue, bool bStrict = true)
	{

		___NODE_TYPE___* pNewNode = m_oNodePool.New();
		
		pNewNode->SetKeyValue(oKey, oValue);

		bool bSucc = m_oDummyRootNode.AddNode(*pNewNode, bStrict);

		if (!bSucc) {
			m_oNodePool.Delete(pNewNode);
		}
	
		return bSucc;

	}

	void Remove(const _Key& oKey)
	{
		___NODE_TYPE___* pNode = m_oDummyRootNode.FindKey(oKey);
		
		// Remove me from tree
		if (pNode != NULL) {
			pNode->RemoveThis();
			m_oNodePool.Delete(pNode);
		}
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Search functions
	//-----------------------------------------------------------------------------------------------------------------	
	
	___NODE_TYPE___* FindKey(const _Key& oKey)
	{
		return m_oDummyRootNode.FindKey(oKey);
	}

	___NODE_TYPE___* FindKeyMin(const _Key& oKey)
	{
		return m_oDummyRootNode.FindKeyMin(oKey);
	}

	___NODE_TYPE___* FindKeyMax(const _Key& oKey)
	{
		return m_oDummyRootNode.FindKeyMax(oKey);
	}

	___NODE_TYPE___* FindSmallest()
	{
		return m_oDummyRootNode.FindSmallest();
	}
	
	___NODE_TYPE___* FindBiggest()
	{
		return m_oDummyRootNode.FindBiggest();
	}

	const ___NODE_TYPE___* FindKey(const _Key& oKey) const
	{
		return m_oDummyRootNode.FindKey(oKey);
	}

	const ___NODE_TYPE___* FindKeyMin(const _Key& oKey) const
	{
		return m_oDummyRootNode.FindKeyMin(oKey);
	}

	const ___NODE_TYPE___* FindKeyMax(const _Key& oKey) const
	{
		return m_oDummyRootNode.FindKeyMax(oKey);
	}

	const ___NODE_TYPE___* FindSmallest() const
	{
		return m_oDummyRootNode.FindSmallest();
	}
	
	const ___NODE_TYPE___* FindBiggest() const
	{
		return m_oDummyRootNode.FindBiggest();
	}
	//-----------------------------------------------------------------------------------------------------------------
	//	Iterator access
	//-----------------------------------------------------------------------------------------------------------------		

	typename ___ITERATOR_TYPE___ Find(const _Key& oKey)
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindKey(oKey));
	}

	typename ___CONST_ITERATOR_TYPE___ Find(const _Key& oKey) const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindKey(oKey));
	}

	typename ___ITERATOR_TYPE___ FindMin(const _Key& oKey)
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMin(oKey));
	}

	typename ___CONST_ITERATOR_TYPE___ FindMin(const _Key& oKey) const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMin(oKey));
	}

	typename ___ITERATOR_TYPE___ FindMax(const _Key& oKey)
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMax(oKey));
	}

	typename ___CONST_ITERATOR_TYPE___ FindMax(const _Key& oKey) const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindKeyMax(oKey));
	}

	typename ___ITERATOR_TYPE___ FindFirst()
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindSmallest());
	}

	typename ___CONST_ITERATOR_TYPE___ FindFirst() const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindSmallest());
	}

	typename ___ITERATOR_TYPE___ FindLast()
	{
		return ___ITERATOR_TYPE___(*m_oDummyRootNode.FindBiggest());
	}

	typename ___CONST_ITERATOR_TYPE___ FindLast() const
	{
		return ___CONST_ITERATOR_TYPE___(*m_oDummyRootNode.FindBiggest());
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Enumerating/iterating tree
	//-----------------------------------------------------------------------------------------------------------------		

	void BeginEnumerateNodes()
	{
		assert(!m_bEnumerating);
		m_oIt = m_oDummyRootNode.FindSmallest();

		m_bEnumerating = true;
	}

	bool IsEnd()
	{
		assert(m_bEnumerating);
		return m_oIt.IsValid();
	}

	___NODE_TYPE___* GetNextNode()
	{
		assert(m_bEnumerating);
		return m_oIt++;
	}

	void EndEnumerateNodes()
	{
		assert(m_bEnumerating);
		m_oIt.Clear();

		m_bEnumerating = false;
	}

	//-----------------------------------------------------------------------------------------------------------------
	//	Direct tree access
	//-----------------------------------------------------------------------------------------------------------------	

	___NODE_TYPE___* operator->()
	{
		return &m_oDummyRootNode;
	}

	const ___NODE_TYPE___* operator->() const
	{
		return &m_oDummyRootNode;
	}

	virtual ___NODE_TYPE___* CreateNode() {

		return m_oNodePool.New();
	}

	virtual void DeleteNode(___NODE_TYPE___* pNode) {

		return m_oNodePool.Delete(pNode);
	}

private:

	typename ___NODE_TYPE___								m_oDummyRootNode;
	typename ___ITERATOR_TYPE___							m_oIt;

	static pool_type/*glibMemPool<node_type, 1024, 16>*/	m_oNodePool;

	bool													m_bIterating;

};

#undef ___ITERATOR_TYPE___ 
#undef ___CONST_ITERATOR_TYPE___ 
#undef ___NODE_TYPE___

#endif //_GLIBUTIL_AVLTREE_H_HEADER_INCLUDED_

