///////////////////////////////////////////////////////////////////////////////////////////////////
//         glibUtil_AVLNode.h
///////////////////////////////////////////////////////////////////////////////////////////////////

/*! \class        glibUtil_AVLNode.h
*  \brief        <brief description> 
*                <long description> 
*  \version      1.0 
*  \date         20.01.2005
*  \author       responsible: [kt]
*/



#ifndef _GLIBUTIL_AVLNODE_H_HEADER_INCLUDED_
#define _GLIBUTIL_AVLNODE_H_HEADER_INCLUDED_
 
#define ___NODE_TYPE___ glibUtilAVLNode<_Key, _Value, _Comperator>

template <class _Key, 
		  class _Value, 
		  class _Comperator>
class glibUtilAVLNode;

template <class _Key, 
		  class _Value, 
		  class _Comperator = glibAvlNode_StdComperator<_Key> >
class glibUtilAVLTree_NodeCreator {

public:

	virtual ___NODE_TYPE___* CreateNode() = NULL;
	virtual void DeleteNode(___NODE_TYPE___* pNode) = NULL;

};

template<class _Key> 
class glibAvlNode_StdComperator 
{
public:

	bool Less(const _Key& oLeft, const _Key& oRight) const
	{

		return oLeft < oRight;
	}

	bool Greater(const _Key& oLeft, const _Key& oRight) const
	{
		return oRight < oLeft;
	}

	bool LessEquval(const _Key& oLeft, const _Key& oRight) const
	{
		if (Equval(oLeft, oRight))
			return true;

		return Less(oLeft, oRight);
	}

	bool GreaterEquval(const _Key& oLeft, const _Key& oRight) const
	{
		if (Equval(oLeft, oRight))
			return true;

		return Less(oRight, oLeft);
	}

	bool Equval(const _Key& oLeft, const _Key& oRight) const
	{
		if (Less(oLeft, oRight))
			return false;

		if (Less(oRight, oLeft))
			return false;

		return true;
	}

	bool NotEquval(const _Key& oLeft, const _Key& oRight) const
	{
		return !(Equval(oLeft, oRight));
	}

};



template<class _Key,
		 class _Value,
		 class _Comperator = glibAvlNode_StdComperator> 

class glibUtilAVLNode 
{
public:

	glibUtilAVLNode()
	{
		m_unProperties = 0;	
		m_pParent	= NULL;
		m_pLeft		= NULL;
		m_pRight	= NULL;

		SetBalance(0);
	}

	glibUtilAVLNode(const _Key& oKey, const _Value& oValue) :
		m_oKey(oKey),
		m_oValue(oValue)
	{
		m_unProperties = 0;	
		m_pParent	= NULL;
		m_pLeft		= NULL;
		m_pRight	= NULL;
		
		SetBalance(0);
	}

	void Delete(glibUtilAVLTree_NodeCreator<_Key, _Value, _Comperator>& oCreator) {

		if (m_pLeft != NULL)
			m_pLeft->Delete(oCreator);

		if (m_pRight != NULL)
			m_pRight->Delete(oCreator);

		// Delete myself
		oCreator.DeleteNode(this);
		// Don't add code here !!! this is not valid anymore at this place !!!!
		// The object just deleted itself !!

	}

	void DeleteChildren(glibUtilAVLTree_NodeCreator<_Key, _Value, _Comperator>& oCreator) {

		if (m_pLeft != NULL)
			m_pLeft->Delete(oCreator);

		if (m_pRight != NULL)
			m_pRight->Delete(oCreator);

		m_pLeft = NULL;
		m_pRight = NULL;


	}

	void Clone(const ___NODE_TYPE___& oOther, glibUtilAVLTree_NodeCreator<_Key, _Value, _Comperator>& oCreator, ___NODE_TYPE___* pParent = NULL)
	{

		m_unProperties = oOther.m_unProperties;
		m_oKey = oOther.m_oKey;
		m_oValue = oOther.m_oValue;
	
		m_pParent = pParent;

		if (oOther.m_pLeft) {

			m_pLeft = oCreator.CreateNode();
			m_pLeft->Clone(*(oOther.m_pLeft), oCreator, this);
		}
		else {
			m_pLeft = NULL;
		}

		if (oOther.m_pRight) {

			m_pRight = oCreator.CreateNode();
			m_pRight->Clone(*(oOther.m_pRight), oCreator, this);
		}
		else {
			m_pRight = NULL;
		}

	}

	void operator=(const ___NODE_TYPE___& oOther)
	{
		this->m_oClient = oOther.m_oClient;
		this->m_pLeft = oOther.m_pLeft;
		this->m_pRight = oOther.m_pRight;
		this->m_pParent = oOther.m_pParent;
		this->m_unProperties = oOther.m_unProperties;

	}

	virtual ~glibUtilAVLNode()
	{
		// Nothing to destroy
		m_unProperties = 0;	
		m_pParent	= NULL;
		m_pLeft		= NULL;
		m_pRight	= NULL;
		
		SetBalance(0);	
		
	}

	void SetKeyValue(const _Key& oKey, const _Value& oValue) {

		// Key value pairs may only be changed if node is not assigned to a tree
		assert(m_pParent == NULL);
		assert(m_pLeft == NULL);
		assert(m_pRight == NULL);

		m_oKey = oKey;
		m_oValue = oValue;

	}

	void SetDummyRoot()
	{
		m_unProperties |= 0x00000001;
	}

	bool AddNode(___NODE_TYPE___& oNode, bool bStrict = true)
	{

		if (IsDummyRoot()) {// If node is the root dummy node 
			
			if (m_pLeft == NULL) {
				
				AddLeft(&oNode);
				DecBalance();
				
			}
			else {
				
				return m_pLeft->AddNode(oNode, bStrict);
				
			}		
			
			return true;
		}
		
		if (m_oComperator.Less(oNode.GetKey(), GetKey())) {
			
			if (m_pLeft == NULL) {
				
				AddLeft(&oNode);
				DecBalance();
				
				UpwardAdd(-1);
				
			}
			else {
				
				return m_pLeft->AddNode(oNode, bStrict);
				
			}
			
		}
		else if (m_oComperator.Greater(oNode.GetKey(), GetKey())) {
			
			if (m_pRight == NULL) {
				
				AddRight(&oNode);
				IncBalance();
				
				UpwardAdd(1);
				
			}
			else {
				
				return m_pRight->AddNode(oNode, bStrict);
				
			}
		}
		else if (!bStrict) {

			// If no strict monotony is specified, add it as smaller node.
			if (m_pLeft == NULL) {
				
				AddLeft(&oNode);
				DecBalance();
				
				UpwardAdd(-1);
				
			}
			else {
				
				return m_pLeft->AddNode(oNode, bStrict);
				
			}
			
		}
		else return false;		// Keys are equal and we are strictly adding keys !! !!

		return true;
	}

	void RemoveThis()
	{
		unsigned int unCase = 0;
		
		if (m_pRight == NULL)
			unCase |= 1;
		if (m_pLeft == NULL)
			unCase |= 2;
		
		switch(unCase) {
			
		case 0:
			//printf("REM INNER\n");
			RemoveInner();
			
			break;
		case 1:
		case 2:
			//printf("REM ASSYM\n");
			RemoveAssym();
			
			break;
		case 3:
			//printf("REM LEAF\n");
			RemoveLeaf();
			
			break;
		}		
	}
	
	const ___NODE_TYPE___* FindKey(const _Key& oKey) const
	{
		if (IsDummyRoot()) {
		
			if (m_pLeft == NULL)
				return NULL;

			return m_pLeft->FindKey(oKey);
		}
		
		if (m_oComperator.Equval(oKey, m_oKey))
			return this;
		
		if (m_oComperator.Less(oKey, m_oKey)) {
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindKey(oKey);
			
		}
		else if (m_oComperator.Greater(oKey, m_oKey)) {
			
			if (m_pRight == NULL)
				return NULL;
			
			return m_pRight->FindKey(oKey);
		}
		
		assert(false);
		return NULL;
	}

	___NODE_TYPE___* FindKey(const _Key& oKey) 
	{
		if (IsDummyRoot()) {
		
			if (m_pLeft == NULL)
				return NULL;

			return m_pLeft->FindKey(oKey);
		}
		
		if (m_oComperator.Equval(oKey, m_oKey))
			return this;
		
		if (m_oComperator.Less(oKey, m_oKey)) {
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindKey(oKey);
			
		}
		else if (m_oComperator.Greater(oKey, m_oKey)) {
			
			if (m_pRight == NULL)
				return NULL;
			
			return m_pRight->FindKey(oKey);
		}
		
		assert(false);
		return NULL;
	}


	const ___NODE_TYPE___* FindSmallest() const
	{
		if (IsDummyRoot()) {
			
			if (m_pLeft == NULL)
				return NULL;
			
		}

		if (m_pLeft == NULL)
			return this;		

		return m_pLeft->FindSmallest();
	}

	___NODE_TYPE___* FindSmallest()
	{
		if (IsDummyRoot()) {
			
			if (m_pLeft == NULL)
				return NULL;
			
		}

		if (m_pLeft == NULL)
			return this;		

		return m_pLeft->FindSmallest();
	}

	const ___NODE_TYPE___* FindBiggest() const
	{
		if (IsDummyRoot()) {
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindBiggest();	
		}
		
		if (m_pRight == NULL)
			return this;

		return m_pRight->FindBiggest();
	}

	___NODE_TYPE___* FindBiggest()
	{
		if (IsDummyRoot()) {
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindBiggest();	
		}
		
		if (m_pRight == NULL)
			return this;

		return m_pRight->FindBiggest();
	}

	const ___NODE_TYPE___* FindKeyMin(const _Key &oKey, const ___NODE_TYPE___** pBestHit = NULL) const
	{
		if (IsDummyRoot()) {
			
			if (m_pLeft == NULL)
				return NULL;

			const ___NODE_TYPE___* pBest = NULL;
			const ___NODE_TYPE___* pResult = /*&*/(m_pLeft->FindKeyMin(oKey, &pBest));
			
			if (pResult == NULL) {
				return pBest;
			}
			
			return pResult;
		}
		
		if (m_oComperator.Equval(oKey, m_oKey))
			return this;
		
		if (m_oComperator.Less(oKey, m_oKey)) {
			
			assert(pBestHit != NULL);
			*pBestHit = this;
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindKeyMin(oKey, pBestHit);
			
		}
		else if (m_oComperator.Greater(oKey, m_oKey)) {
			
			if (m_pRight == NULL)
				return NULL;
			
			return m_pRight->FindKeyMin(oKey, pBestHit);
		}
		
		assert(false);
		return NULL;
	}

	___NODE_TYPE___* FindKeyMin(const _Key &oKey, ___NODE_TYPE___** pBestHit = NULL) 
	{
		if (IsDummyRoot()) {
			
			if (m_pLeft == NULL)
				return NULL;

			___NODE_TYPE___* pBest = NULL;
			___NODE_TYPE___* pResult = /*&*/(m_pLeft->FindKeyMin(oKey, &pBest));
			
			if (pResult == NULL) {
				return pBest;
			}
			
			return pResult;
		}
		
		if (m_oComperator.Equval(oKey, m_oKey))
			return this;
		
		if (m_oComperator.Less(oKey, m_oKey)) {
			
			assert(pBestHit != NULL);
			*pBestHit = this;
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindKeyMin(oKey, pBestHit);
			
		}
		else if (m_oComperator.Greater(oKey, m_oKey)) {
			
			if (m_pRight == NULL)
				return NULL;
			
			return m_pRight->FindKeyMin(oKey, pBestHit);
		}
		
		assert(false);
		return NULL;
	}


	const ___NODE_TYPE___* FindKeyMax(const _Key& oKey, const ___NODE_TYPE___** pBestHit = NULL) const
	{
		if (IsDummyRoot()) {

			if (m_pLeft == NULL)
				return NULL;

			const ___NODE_TYPE___* pBest = NULL;
			const ___NODE_TYPE___* pResult = /*&*/(m_pLeft->FindKeyMax(oKey, &pBest));
			
			if (pResult == NULL) {
				return pBest;
			}
			
			return pResult;
		}
		
		if (m_oComperator.Equval(oKey, m_oKey))
			return this;
		
		if (m_oComperator.Less(oKey, m_oKey)) {
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindKeyMax(oKey, pBestHit);
			
		}
		else if (m_oComperator.Greater(oKey, m_oKey)) {
			
			assert(pBestHit != NULL);
			*pBestHit = this;
			
			if (m_pRight == NULL)
				return NULL;
			
			return m_pRight->FindKeyMax(oKey, pBestHit);
		}
		
		assert(false);
		return NULL;		
	}

	___NODE_TYPE___* FindKeyMax(const _Key& oKey, ___NODE_TYPE___** pBestHit = NULL) 
	{
		if (IsDummyRoot()) {

			if (m_pLeft == NULL)
				return NULL;

			___NODE_TYPE___* pBest = NULL;
			___NODE_TYPE___* pResult = (m_pLeft->FindKeyMax(oKey, &pBest));
			
			if (pResult == NULL) {
				return pBest;
			}
			
			return pResult;
		}
		
		if (m_oComperator.Equval(oKey, m_oKey))
			return this;
		
		if (m_oComperator.Less(oKey, m_oKey)) {
			
			if (m_pLeft == NULL)
				return NULL;
			
			return m_pLeft->FindKeyMax(oKey, pBestHit);
			
		}
		else if (m_oComperator.Greater(oKey, m_oKey)) {
			
			assert(pBestHit != NULL);
			*pBestHit = this;
			
			if (m_pRight == NULL)
				return NULL;
			
			return m_pRight->FindKeyMax(oKey, pBestHit);
		}
		
		assert(false);
		return NULL;		
	}


	// Client access

	_Value* operator->()
	{
		return &m_oValue;
	}

	const _Value* operator->() const
	{
		return &m_oValue;
	}

	_Key& GetKey()
	{
		return m_oKey;
	}

	_Value& GetValue()
	{
		return m_oValue;
	}

	const _Key& GetKey() const
	{
		return m_oKey;
	}

	const _Value& GetValue() const
	{
		return m_oValue;
	}

	void CountDepth(int& nMax, int& nMin, int nGeneration) const
	{
		
		if (IsDummyRoot()) {
			
			nMax = 0;
			nMin = 0;
			if (m_pLeft == NULL)
				return;
			m_pLeft->CountDepth(nMax, nMin, 0);
			return;
			
		}
		
		int nLeftMin = nGeneration;
		int nLeftMax = nGeneration;
		
		if (m_pLeft != 0) {
			m_pLeft->CountDepth(nLeftMax, nLeftMin, nGeneration + 1);
		}
		
		int nRightMin = nGeneration;
		int nRightMax = nGeneration;
		
		if (m_pRight != 0) {
			m_pRight->CountDepth(nRightMax, nRightMin, nGeneration + 1);
		}
		
		nMax = nRightMax;
		if (nLeftMax > nMax)
			nMax = nLeftMax;
		
		nMin = nRightMin;
		if (nLeftMin < nMin)
			nMin = nLeftMin;
		
		if (nRightMax - nLeftMax != GetBalance()) {
			
			assert(false);
			//printf("** !!!!! ** Misbalanced at node (%d, %d) \n\n" , nRightMax - nLeftMax, GetBalance());
			//__asm int 3;
		}
		
	}

	const ___NODE_TYPE___* FindNextKey() const
	{
	
		if (m_pRight == NULL) {

			return FindNextKeyUpward(this);
		}
		else 
		{
			return FindNext();
		}


	}

	___NODE_TYPE___* FindNextKey() 
	{
	
		if (m_pRight == NULL) {

			return FindNextKeyUpward(this);
		}
		else 
		{
			return FindNext();
		}


	}

	const ___NODE_TYPE___* FindPrevKey() const
	{
		if (m_pLeft == NULL) {
			
			return FindPrevKeyUpward(this);
		}
		else 
		{
			return FindPrev();
		}
	}	

	___NODE_TYPE___* FindPrevKey() 
	{
		if (m_pLeft == NULL) {
			
			return FindPrevKeyUpward(this);
		}
		else 
		{
			return FindPrev();
		}
	}	

	class iterator {

	public:
		iterator()
		{
			m_pPresent = NULL;
		}

		iterator(___NODE_TYPE___& pNode)
		{
			m_pPresent = &pNode;
		}

		~iterator()
		{
		}

		void operator=(___NODE_TYPE___* pNode)
		{
			m_pPresent = pNode;
		}

		void operator=(typename ___NODE_TYPE___::iterator& oOther)
		{
			m_pPresent = oOther.m_pPresent;
		}

		___NODE_TYPE___& operator++()
		{
			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindNextKey();
			}

			return *m_pPresent;
		}

		___NODE_TYPE___& operator--()
		{
			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindPrevKey();
			}		
			
			return *m_pPresent;
		}

		___NODE_TYPE___& operator++(int nDummy)
		{
			___NODE_TYPE___* pOld = m_pPresent;


			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindNextKey();
			}
			return *pOld;
		}
		
		___NODE_TYPE___& operator--(int nDummy)
		{
			___NODE_TYPE___* pOld = m_pPresent;

			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindPrevKey();
			}		
			
			return *pOld;
		}		

		bool IsValid() const{

			if (m_pPresent != NULL)
				return true;

			return false;
		}

		_Key& GetKey() {
			return m_pPresent->GetKey();
		}

		_Value& GetValue() {
			return m_pPresent->GetValue();
		}


		___NODE_TYPE___* operator->() 
		{
			return &m_pPresent;
		}

		void Clear()
		{
			m_pPresent = NULL;
		}

	private:

		___NODE_TYPE___* m_pPresent;
	};

	class const_iterator {

	public:
		const_iterator()
		{
			m_pPresent = NULL;
		}

		const_iterator(const ___NODE_TYPE___& pNode)
		{
			m_pPresent = &pNode;
		}

		const_iterator(___NODE_TYPE___& pNode)
		{
			m_pPresent = &pNode;
		}

		~const_iterator()
		{
		}

		void operator=(const ___NODE_TYPE___* pNode)
		{
			m_pPresent = pNode;
		}

		void operator=(___NODE_TYPE___* pNode)
		{
			m_pPresent = pNode;
		}

		void operator=(const typename ___NODE_TYPE___::iterator& oOther)
		{
			m_pPresent = oOther.m_pPresent;
		}

		void operator=(typename ___NODE_TYPE___::iterator& oOther)
		{
			m_pPresent = oOther.m_pPresent;
		}

		const ___NODE_TYPE___& operator++()
		{
			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindNextKey();
			}

			return *m_pPresent;
		}

		const ___NODE_TYPE___& operator--()
		{
			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindPrevKey();
			}		
			
			return *m_pPresent;
		}

		const ___NODE_TYPE___& operator++(int nDummy)
		{
			___NODE_TYPE___* pOld = m_pPresent;


			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindNextKey();
			}
			return *pOld;
		}
		
		const ___NODE_TYPE___& operator--(int nDummy)
		{
			___NODE_TYPE___* pOld = m_pPresent;

			if (m_pPresent != NULL) {
				m_pPresent = m_pPresent->FindPrevKey();
			}		
			
			return *pOld;
		}		

		bool IsValid() const{

			if (m_pPresent != NULL)
				return true;

			return false;
		}

		const _Key& GetKey() const {
			return m_pPresent->GetKey();
		}

		const _Value& GetValue() const {
			return m_pPresent->GetValue();
		}


		const ___NODE_TYPE___* operator->() const
		{
			return &m_pPresent;
		}

		void Clear()
		{
			m_pPresent = NULL;
		}

	private:

		const ___NODE_TYPE___* m_pPresent;
	};

protected:

	//---------------------------------------------------------------------------------------------------------------------

	// Helper functions

	void AddToParent(bool bLeft,
					 bool bRight, 
					 ___NODE_TYPE___* pSelf,
					 ___NODE_TYPE___* pParent)
	{
		if (bLeft) {
			pParent->m_pLeft = pSelf;
			pSelf->m_pParent = pParent;
			pSelf->SetLeftSon();
		} else if (bRight) {
			pParent->m_pRight = pSelf;
			pSelf->m_pParent = pParent;
			pSelf->SetRightSon();
		} else {
			assert(false);
		}
	}

	void AddToParentOnly(bool bLeft, 
						 bool bRight, 
						 ___NODE_TYPE___* pParent,
						 ___NODE_TYPE___* pNode) 
	{
		if (bLeft) {
			pParent->m_pLeft = pNode;
			pNode->SetLeftSon();
		} else if (bRight) {
			pParent->m_pRight = pNode;
			pNode->SetRightSon();
		} else {
			debugASSERT(false);
		}
		
	}


	void SafeAddLeftParentOnly(___NODE_TYPE___* pLeft,
							   ___NODE_TYPE___* pNode)
	{
		if (pLeft != NULL) {
			pLeft->m_pParent = pNode;
			pLeft->SetLeftSon();
		}
	}
	
	void SafeAddRightParentOnly(___NODE_TYPE___* pRight,
								___NODE_TYPE___* pNode)
	{
		if (pRight != NULL) {
			pRight->m_pParent = pNode;
			pRight->SetRightSon();
		}
	}
	
	
	void SafeAddLeft(___NODE_TYPE___* pLeft,
					 ___NODE_TYPE___* pNode)
	{
		pNode->m_pLeft = pLeft;
		if (pLeft != NULL) {
			pLeft->m_pParent = pNode;
			pLeft->SetLeftSon();
		}
	}
	
	void SafeAddRight(___NODE_TYPE___* pRight,
					  ___NODE_TYPE___* pNode)
	{
		pNode->m_pRight = pRight;
		if (pRight != NULL) {
			pRight->m_pParent = pNode;
			pRight->SetRightSon();
		}
	}
		
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	

	void AddLeft(___NODE_TYPE___* pNewNode)
	{
		if (pNewNode == NULL) {
			if (m_pLeft != NULL) {
				m_pLeft->m_pParent = NULL;
				m_pLeft->SetNoSon();
			}
			
			m_pLeft = NULL;
			
		}
		else {
			
			m_pLeft = pNewNode;
			m_pLeft->m_pParent = this;
			m_pLeft->SetLeftSon();
		}
	}

	void AddRight(___NODE_TYPE___* pNewNode)
	{
		if (pNewNode == NULL) {
			if (m_pRight != NULL) {
				m_pRight->m_pParent = NULL;
				m_pRight->SetNoSon();
			}
			
			m_pRight = NULL;
			
		}
		else {
			
			m_pRight = pNewNode;
			m_pRight->m_pParent = this;
			m_pRight->SetRightSon();
		}
	}
	
	void RemoveFromTree()
	{
		if (this == NULL)
			return;
		
		if (m_pParent != NULL) {
			
			if (IsLeftSon()) {
				m_pParent->AddLeft(NULL);
			} 
			else if (IsRightSon()) {
				m_pParent->AddRight(NULL);
			}
			
			
		}
		
	}

	void CompletelyRemoveFromTree()
	{
		if (this == NULL)
			return;
		
		// Remove my left son
		
		if (m_pLeft != NULL) {
			AddLeft(NULL);
		}
		
		// Remove my right son
		
		if (m_pRight != NULL) {
			AddRight(NULL);
		}
		
		// Remove me from parent
		
		if (m_pParent != NULL) {
			
			if (IsLeftSon()) {
				m_pParent->AddLeft(NULL);
			} 
			else if (IsRightSon()) {
				m_pParent->AddRight(NULL);
			}
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	void RemoveInner()
	{
		// We are swapping this node with our next node. All structure relevant properties are swapped aswell
		// After all we remove this from the tree

		___NODE_TYPE___* pNextNode = NULL;

		// In order to keep the removal of nodes randomly balanced, we take a random number to choose wheater
		// we replace our node with the next or the previous key.

		int nRand;
		int nMaxRand;

		pNextNode = FindNext();

		assert(pNextNode);

		// Save next node's properties
		___NODE_TYPE___* pNextParent		= pNextNode->m_pParent;
		___NODE_TYPE___* pNextLeft		= pNextNode->m_pLeft;
		___NODE_TYPE___* pNextRight		= pNextNode->m_pRight;

		int nNextBalance		= pNextNode->GetBalance();
		bool bNextLeft			= pNextNode->IsLeftSon();
		bool bNextRight			= pNextNode->IsRightSon();

		// Now replace this with next node
		
		___NODE_TYPE___* pParent			= m_pParent;
		___NODE_TYPE___* pLeft			= m_pLeft;
		___NODE_TYPE___* pRight			= m_pRight;

		int nBalance			= GetBalance();
		bool bLeft				= IsLeftSon();
		bool bRight				= IsRightSon();

		//
		//	We are swapping all structure relevant parameters now.
		//	This depends weather the next node is a direct child of this or not.
		//


		// Swap balances
		SetBalance(nNextBalance);
		pNextNode->SetBalance(nBalance);


		// Add next node to my parent node
		AddToParent(bLeft, bRight, pNextNode, pParent);

		// If the next node is our direct neighbour

		// Right neigbour
		if (pNextNode == m_pRight) {

			//printf("CASE R\n");

			pNextNode->m_pRight = this;
			pNextNode->m_pLeft = pLeft;
			pNextNode->m_pParent = pParent;

			m_pRight = pNextRight;
			m_pLeft = pNextLeft;
			m_pParent = pNextNode;

			SafeAddLeftParentOnly(pLeft, pNextNode);
			SafeAddLeftParentOnly(pNextLeft, this);
			SafeAddRightParentOnly(pNextRight, this);

			SetRightSon();
			RemoveThis();

			return;
		}

		// Left neigbour
		if (pNextNode == m_pLeft) {

			//printf("CASE L\n");

			pNextNode->m_pLeft = this;
			pNextNode->m_pRight = pRight;
			pNextNode->m_pParent = pParent;

			m_pRight = pNextRight;
			m_pLeft = pNextLeft;
			m_pParent = pNextNode;

			SafeAddRightParentOnly(pRight, pNextNode);
			SafeAddLeftParentOnly(pNextLeft, this);
			SafeAddRightParentOnly(pNextRight, this);

			SetLeftSon();
			RemoveThis();

			return;
		}

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		//printf("CASE D\n");

		// Normal behaviour

		SafeAddLeft(pLeft, pNextNode);
		SafeAddRight(pRight, pNextNode);

		// Replace next with this
		
		AddToParent(bNextLeft, bNextRight, this, pNextParent);


		SafeAddLeft(pNextLeft, this);
		SafeAddRight(pNextRight, this);

		// Remove me after taking the position of next node.
		RemoveThis();
	}
	
	void RemoveAssym()
	{
		___NODE_TYPE___* pParent = m_pParent;
		
		bool bLeft = IsLeftSon();
		bool bRight = IsRightSon();
		
		if (m_pRight != NULL) {
			
			if (IsRightSon()) {
				
				pParent->AddRight(m_pRight);
				
			}
			else if (IsLeftSon()) {
				
				pParent->AddLeft(m_pRight);
				
			}
			else assert(false);
			
			
		}
		else if (m_pLeft != NULL) {
			
			if (IsRightSon()) {
				
				pParent->AddRight(m_pLeft);
				
			}
			else if (IsLeftSon()) {
				
				pParent->AddLeft(m_pLeft);
				
			}
			else assert(false);
			
		}
		else assert(false);
		
		// Adapt parent's balance and recourse upward, stop recoursion at top dummy node.
		if (!(pParent->IsDummyRoot())) {
			
			if (bRight) {
				
				pParent->DecBalance();
				pParent->UpwardRemove(-1);
			}
			else if (bLeft) {
				
				pParent->IncBalance();
				pParent->UpwardRemove(1);
			}
		}
	}

	void RemoveLeaf()
	{
		// Special case:
		
		// If this is the only interval in the tree simply remove it !!!!
		if (m_pParent->IsDummyRoot()) {
			m_pParent->AddLeft(NULL);
			return;
		}
		
		// Common case:
		
		___NODE_TYPE___* pParent = m_pParent;
		
		if (IsLeftSon()) {
			
			pParent->AddLeft(NULL);
			
			pParent->IncBalance();
			
			pParent->UpwardRemove(1);
			
		}
		else if (IsRightSon()) {
			
			pParent->AddRight(NULL);
			
			pParent->DecBalance();
			
			pParent->UpwardRemove(-1);
			
		}
		else assert(false);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	void UpwardAdd(int nAddBalance)
	{
		if (IsDummyRoot())
			return;

		___NODE_TYPE___* pParent = m_pParent;

		switch(nAddBalance) {
		case -1:

			switch(GetBalance()) {

			case 0:	// Nothing to do ! Subtree did not grow
				break;

			case -1: // Left side of tree grew by 1.

				// Balance is still in valid range, but our parent's balance could be invalid:

				// Adapt parent's balance and recourse upward 
				if (IsLeftSon()) {
					
					pParent->DecBalance();

					pParent->UpwardAdd(-1);

				} else if (IsRightSon()) {

					pParent->IncBalance();

					pParent->UpwardAdd(1);

				}
				else 
					assert(false);

				break;
			case -2:

				int nBalChild = m_pLeft->GetBalance();

				switch(nBalChild) {

				case -1:

					// Adapt balances
					SetBalance(0);
					m_pLeft->SetBalance(0);

					// And change structure in order to balance the tree
					RotateRight();

					break;

				case 1:

					{
					
						// Adapt node balances

						int nGrandChildBalance = m_pLeft->m_pRight->GetBalance();

						m_pLeft->m_pRight->SetBalance(0);

						if (nGrandChildBalance > 0) {

							SetBalance(0);		
							m_pLeft->SetBalance(-1);
							

						}
						else if (nGrandChildBalance < 0) {

							SetBalance(1);
							m_pLeft->SetBalance(0);

						}
						else {
							SetBalance(0);
							m_pLeft->SetBalance(0);
						}
							

						// And fix structure in order to balance tree

						RotateLeftRight();
					}

					break;

				default:
					assert(false);
				}
				break;	

			}
			break;
		case 1:

			switch(GetBalance()) {

			case 0:	// Nothing to do ! Subtree did not grow
				break;

			case 1: // Right side of tree grew by 1.
				// Balance is still in valid range, but our parent's balance could be invalid:

				// Adapt parent's balance and recourse upward 
				if (IsLeftSon()) {
					
					pParent->DecBalance();

					pParent->UpwardAdd(-1);

				} else if (IsRightSon()) {

					pParent->IncBalance();

					pParent->UpwardAdd(1);

				}
				else 
					assert(false);

				break;
			case 2:

				int nBalChild = m_pRight->GetBalance();

				switch(nBalChild) {

				case 1:

					// Adapt balances
					SetBalance(0);
					m_pRight->SetBalance(0);

					RotateLeft();

					break;

				case -1:

					{
					
						// Adapt node balances

						int nGrandChildBalance = m_pRight->m_pLeft->GetBalance();


						m_pRight->m_pLeft->SetBalance(0);

						if (nGrandChildBalance > 0) {

							SetBalance(-1);
							m_pRight->SetBalance(0);

						}
						else if (nGrandChildBalance < 0) {

							SetBalance(0);
							m_pRight->SetBalance(1);

						}
						else {

							SetBalance(0);
							m_pRight->SetBalance(0);

						}
							

						// And fix structure in order to balance tree

						RotateRightLeft();
					}
					break;

				default:
					assert(false);
				}
				

			}		

		}
	}

	void UpwardRemove(int nRemBalance)
	{
		___NODE_TYPE___* pParent = m_pParent;

		bool bLeft = IsLeftSon();
		bool bRight = IsRightSon();

		//printf("Recourse Upward  for %d\n", m_nKey);

		if (IsDummyRoot())
			return;

		bool bRecourse = false;

		switch(nRemBalance) {
			
		case -1:	// We removed node on the right subtree

		
			switch(GetBalance()) {

			case 0:

				bRecourse = true;

				break;
			case -1:// Nothing to do anymore

				break;
			case -2:
				// We have to restructure the tree
				{
 					int nChildBalance = m_pLeft->GetBalance();

					switch(nChildBalance) {
					case 0:

						m_pLeft->SetBalance(1);
						SetBalance(-1);

						RotateRight();
						break;
					case -1:

						m_pLeft->SetBalance(0);
						SetBalance(0);

						RotateRight();

						bRecourse = true;

						break;

					case 1:
						{

							int nBalGrandchild = m_pLeft->m_pRight->GetBalance();

							m_pLeft->m_pRight->SetBalance(0);
								
							switch(nBalGrandchild) {
							case 1:

								SetBalance(0);
								m_pLeft->SetBalance(-1);
								break;
							case 0:

								SetBalance(0);
								m_pLeft->SetBalance(0);
								break;

							case -1:

								SetBalance(1);
								m_pLeft->SetBalance(0);
								break;
							}

							RotateLeftRight();
							bRecourse = true;
						}

					}
				}
				break;
			default:
				assert(false);
			}

			break;
		case 1:		// We removed node on the left subtree

			
			switch(GetBalance()) {
			case 0:

				bRecourse = true;

				break;
			case 1:// Nothing to do anymore

				break;
			case 2:
				// We have to restructure the tree
				{


					int nChildBalance = m_pRight->GetBalance();

					switch(nChildBalance) {
					case 0:

						m_pRight->SetBalance(-1);
						SetBalance(1);

						RotateLeft();
						break;
					case 1:

						m_pRight->SetBalance(0);
						SetBalance(0);

						RotateLeft();

						bRecourse = true;

						break;

					case -1:
						{

							int nBalGrandchild = m_pRight->m_pLeft->GetBalance();

							m_pRight->m_pLeft->SetBalance(0);

							switch(nBalGrandchild) {
							case -1:

								SetBalance(0);
								m_pRight->SetBalance(1);
								break;
							case 0:

								SetBalance(0);
								m_pRight->SetBalance(0);
								break;

							case 1:

								SetBalance(-1);
								m_pRight->SetBalance(0);
								break;
							}

							RotateRightLeft();
							bRecourse = true;
						}
					}
				}
				break;

			default:
				assert(false);
			}


			break;
		case 0:		// Bad case !
			assert(false);	
			break;
		}

		// recourse upward

		if (bRecourse) {

			if (bLeft) {
				pParent->IncBalance();
				pParent->UpwardRemove(1);
			}
			else if (bRight) {

				pParent->DecBalance();
				pParent->UpwardRemove(-1);
			}
			else assert(false);
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	

	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	void RotateRight()
	{
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		___NODE_TYPE___* pParent	= m_pParent;
		___NODE_TYPE___* pLeft		= m_pLeft;
			
		___NODE_TYPE___* pSubtree1	= m_pLeft->m_pLeft;
		___NODE_TYPE___* pSubtree2	= m_pLeft->m_pRight;
		___NODE_TYPE___* pSubtree3	= m_pRight;
		
		// Remove all subtrees
		pSubtree1->RemoveFromTree();
		pSubtree2->RemoveFromTree();
		pSubtree3->RemoveFromTree();
		
		// Remove all nodes from tree
		
		bool bIsLeft = IsLeftSon();
		bool bIsRight = IsRightSon();
		
		// Add my left son as root
		
		AddToParent(bIsLeft, bIsRight, pLeft, pParent);
		
		SafeAddLeft(pSubtree2, this); 
		SafeAddRight(pSubtree3, this); 
		
		// Add me as right son of my ex son
		pLeft->m_pRight = this;
		m_pParent = pLeft;
		SetRightSon();
		
		// Assign subtrees
		
		SafeAddLeft(pSubtree1, pLeft); 
	}

	void RotateLeft()
	{
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		___NODE_TYPE___*  pParent = m_pParent;
		___NODE_TYPE___*  pRight = m_pRight;
		
		___NODE_TYPE___*  pSubtree1 = m_pLeft;
		___NODE_TYPE___*  pSubtree2 = m_pRight->m_pLeft;
		___NODE_TYPE___*  pSubtree3 = m_pRight->m_pRight;
		
		// Remove all subtrees
		pSubtree1->RemoveFromTree();
		pSubtree2->RemoveFromTree();
		pSubtree3->RemoveFromTree();
		
		
		// Remove all affected node from tree
		
		bool bIsLeft = IsLeftSon();
		bool bIsRight = IsRightSon();
		
		// Add my left son as root
		
		AddToParent(bIsLeft, bIsRight, pRight, pParent);
		
		SafeAddLeft(pSubtree1, this); 
		SafeAddRight(pSubtree2, this); 
		
		// My forer son is new root node
		
		pRight->m_pLeft = this;
		m_pParent = pRight;
		SetLeftSon();
		
		// Add third subtree to new root node
		
		SafeAddRight(pSubtree3, pRight); 	
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	void RotateRightLeft()
	{
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		___NODE_TYPE___*  pParent = m_pParent;
		___NODE_TYPE___*  pRight = m_pRight;
		___NODE_TYPE___*  pRightLeft = m_pRight->m_pLeft;
		
		___NODE_TYPE___*  pSubtree1 = m_pLeft;
		___NODE_TYPE___*  pSubtree2 = m_pRight->m_pLeft->m_pLeft;
		___NODE_TYPE___*  pSubtree3 = m_pRight->m_pLeft->m_pRight;
		___NODE_TYPE___*  pSubtree4 = m_pRight->m_pRight;
		
		// Remove all affected nodes
		
		bool bIsLeft = IsLeftSon();
		bool bIsRight = IsRightSon();
		
		pSubtree1->RemoveFromTree();
		pSubtree2->RemoveFromTree();
		pSubtree3->RemoveFromTree();
		pSubtree4->RemoveFromTree();
		
		// Add right left greandson as root
		
		AddToParent(bIsLeft, bIsRight, pRightLeft, pParent);
		
		// Add me and right son to new root
		
		pRightLeft->m_pRight = pRight;
		pRight->m_pParent = pRightLeft;
		pRight->SetRightSon();
		
		SafeAddLeft(pSubtree3, pRight);
		SafeAddRight(pSubtree4, pRight);
		
		pRightLeft->m_pLeft = this;
		m_pParent = pRightLeft;
		SetLeftSon();
		
		SafeAddLeft(pSubtree1, this);
		SafeAddRight(pSubtree2, this);		
	}

	void RotateLeftRight()
	{
		___NODE_TYPE___*  pParent = m_pParent;
		___NODE_TYPE___*  pLeft = m_pLeft;
		___NODE_TYPE___*  pLeftRight = m_pLeft->m_pRight;
		
		___NODE_TYPE___*  pSubtree1 = m_pLeft->m_pLeft;
		___NODE_TYPE___*  pSubtree2 = m_pLeft->m_pRight->m_pLeft;
		___NODE_TYPE___*  pSubtree3 = m_pLeft->m_pRight->m_pRight;
		___NODE_TYPE___*  pSubtree4 = m_pRight;
		
		// 
		
		bool bIsLeft = IsLeftSon();
		bool bIsRight = IsRightSon();
		
		pSubtree1->RemoveFromTree();
		pSubtree2->RemoveFromTree();
		pSubtree3->RemoveFromTree();
		pSubtree4->RemoveFromTree();
		
		// Add right left greandson as root
		
		AddToParent(bIsLeft, bIsRight, pLeftRight, pParent);
		
		// Add me and right son to new root
		
		pLeftRight->m_pLeft = pLeft;
		pLeft->m_pParent = pLeftRight;
		pLeft->SetLeftSon();
		
		SafeAddLeft(pSubtree1, pLeft);
		SafeAddRight(pSubtree2, pLeft);
		
		pLeftRight->m_pRight = this;
		m_pParent = pLeftRight;
		SetRightSon();
		
		SafeAddLeft(pSubtree3, this);
		SafeAddRight(pSubtree4, this);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	___NODE_TYPE___* FindNext()
	{
		if (m_pRight != NULL)
			return m_pRight->FindNextRecourse();
		
		return this;
	}
	
	const ___NODE_TYPE___* FindNext() const
	{
		if (m_pRight != NULL)
			return m_pRight->FindNextRecourse();
		
		return this;
	}

	___NODE_TYPE___* FindPrev()
	{
		if (m_pLeft != NULL)
			return m_pLeft->FindPrevRecourse();
		
		return this;
	}	

	const ___NODE_TYPE___* FindPrev() const
	{
		if (m_pLeft != NULL)
			return m_pLeft->FindPrevRecourse();
		
		return this;
	}	

	___NODE_TYPE___* FindNextRecourse()
	{
		if (m_pLeft != NULL)
			return m_pLeft->FindNextRecourse();
		
		return this;
	}

	const ___NODE_TYPE___* FindNextRecourse() const
	{
		if (m_pLeft != NULL)
			return m_pLeft->FindNextRecourse();
		
		return this;
	}

	const ___NODE_TYPE___* FindPrevRecourse() const
	{
		if (m_pRight != NULL)
			return m_pRight->FindPrevRecourse();
		
		return this;
	}


	___NODE_TYPE___* FindPrevRecourse()
	{
		if (m_pRight != NULL)
			return m_pRight->FindPrevRecourse();
		
		return this;
	}

	const ___NODE_TYPE___* FindNextKeyUpward(___NODE_TYPE___* pSon) const
	{
		if (IsDummyRoot())
			return NULL;
		
		if (pSon == m_pLeft)
			return this;
		
		if (m_pParent == NULL)
			return NULL;

		return m_pParent->FindNextKeyUpward(this);
		
	}

	___NODE_TYPE___* FindNextKeyUpward(___NODE_TYPE___* pSon)
	{
		if (IsDummyRoot())
			return NULL;
		
		if (pSon == m_pLeft)
			return this;
		
		if (m_pParent == NULL)
			return NULL;

		return m_pParent->FindNextKeyUpward(this);
		
	}

	const ___NODE_TYPE___* FindPrevKeyUpward(___NODE_TYPE___* pSon) const
	{
		if (IsDummyRoot())
			return NULL;
		
		if (pSon == m_pRight)
			return this;
		
		if (m_pParent == NULL)
			return NULL;

		return m_pParent->FindPrevKeyUpward(this);
	}	

	___NODE_TYPE___* FindPrevKeyUpward(___NODE_TYPE___* pSon)
	{
		if (IsDummyRoot())
			return NULL;
		
		if (pSon == m_pRight)
			return this;
		
		if (m_pParent == NULL)
			return NULL;

		return m_pParent->FindPrevKeyUpward(this);
	}	

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	void SetLeftSon()
	{
		m_unProperties |= 0x00000002;
		m_unProperties &= ~0x00000004;
	}

	void SetRightSon()
	{
		m_unProperties |= 0x00000004;
		m_unProperties &= ~0x00000002;
	}
	
	void SetNoSon() 
	{

		m_unProperties &= ~0x00000006;
		
	}

	void SetBalance(int nBal)
	{
		if (nBal > 2)
			nBal = 2;
		if (nBal < -2)
			nBal = -2;

		nBal += 2;

		m_unProperties &= ~0x000f0000;
		m_unProperties |= ((unsigned int)nBal) << 16;
	}

	void DecBalance() {

		m_unProperties = (((((m_unProperties & 0x000f0000) >> 16) - 1) << 16) & 0x000f0000) | 
						 (m_unProperties & ~0x000f0000);

	}

	void IncBalance() {
		
		m_unProperties = (((((m_unProperties & 0x000f0000) >> 16) + 1) << 16) & 0x000f0000) | 
			(m_unProperties & ~0x000f0000);
		
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	bool IsRightSon() const{
		if (m_unProperties & 0x00000004)
			return true;
		
		return false;
	}

	bool IsLeftSon() const{
		if (m_unProperties & 0x00000002)
			return true;
		
		return false;
	}

	bool IsNoSon() const{

		if ((m_unProperties & 0x00000006) == 0)
			return true;

		return false;

	}

	int GetBalance() const{

		return (int)((m_unProperties >> 16) & 0x0000000f) - 2;
	}

	bool IsDummyRoot() const{
		if (m_unProperties & 0x00000001)
			return true;

		return false;
	}

private:

	___NODE_TYPE___*	m_pParent;
	___NODE_TYPE___*	m_pLeft;
	___NODE_TYPE___*	m_pRight;

	unsigned int										m_unProperties;
	_Key												m_oKey;
	_Value												m_oValue;
	_Comperator											m_oComperator;

};

#undef ___NODE_TYPE___


#endif //_GLIBUTIL_AVLNODE_H_HEADER_INCLUDED_

