///////////////////////////////////////////////////////////////////////////////////////////////////
//         
// (c) CARLADATA GmbH 
//     Landwehrstrasse 30 
//     97249 Eisingen 
//     Germany Tel: 09306/9833-0
//  
//  MiscBuffer.h
//         
///////////////////////////////////////////////////////////////////////////////////////////////////

/*! \class        MiscBuffer.h
*  \brief        <brief description> 
*                <long description> 
*  \version      1.0 
*  \date         15.10.2008
*  \author       responsible: [kt]
*/



#ifndef _MISCBUFFER_H_HEADER_INCLUDED_
#define _MISCBUFFER_H_HEADER_INCLUDED_

enum glibBufferSortingAlgorithm {

	glibBufferSortingAlgorithm_Quick,
	glibBufferSortingAlgorithm_Radix,
	glibBufferSortingAlgorithm_Insertion

};



/*!  \global       template<class _T, int nGrowstep = 1024>
class glibMemBuffer 
*    \file         glibMemBuffer.h
*    \brief        <brief description> 
*                  <long description> 
*/
template<class _T, int nGrowstep = 1024>
class glibMemBuffer {

public:

	glibMemBuffer() {

		m_pBuffer = NULL;
		m_nBufferCap = 0;
		m_nBufferSize = 0;
		m_bLocked = false;

	}

	glibMemBuffer(const glibMemBuffer& oOther) {

		m_pBuffer = NULL;
		m_nBufferCap = 0;
		m_nBufferSize = 0;
		m_bLocked = false;

		*this = oOther;

	}

	inline ~glibMemBuffer() {

		if (m_pBuffer != NULL)
			delete[] m_pBuffer;
	}

	inline void LockBuffer(int nSize) {

		EnsureBufferCapacity(nSize);
		m_nBufferSize = nSize;
		m_bLocked = true;

	}

	inline void LockBuffer() {
		m_bLocked = true;
	}

	inline void Zero(int nSize) {

		EnsureBufferCapacity(nSize);
		
		// Zero out buffer
		memset(m_pBuffer, 0, m_nBufferCap);

	}

	virtual void UnLockBuffer() {
		m_bLocked = false;
	}



	// Garbage collection ..... shrik buffer to required size
	inline void Garbage() {

		int nRequiredBlockSize = ComputeBlockCapacity(m_nBufferSize);

		// Shrink the buffer down to the specified size
		if (nRequiredBlockSize < m_nBufferCap) {
		

			_T* pTempPtr = NULL;

			int nNewSize = 0;

			pTempPtr = m_pBuffer;

			m_pBuffer = new _T[nRequiredBlockSize];

			assert(m_pBuffer != NULL);

			m_nBufferCap = nRequiredBlockSize;

			if (pTempPtr != NULL) {

				if (m_nBufferSize > 0) {

					memcpy(m_pBuffer, pTempPtr, m_nBufferSize * sizeof(_T));

				}
				delete[] pTempPtr;
			}
		}
	}

	inline void SetBufferSize(int nSize)
	{
		

		// Make sure that the allocated buffer is large enough
		EnsureBufferCapacity(nSize);

		m_nBufferSize = nSize;
	}

	inline void SetBufferSizeZero(int nSize)
	{
		int nCurrentBufferSize = m_nBufferSize;		

		if (nCurrentBufferSize < 0)
			nCurrentBufferSize = 0;

		// Make sure that the allocated buffer is large enough
		EnsureBufferCapacity(nSize);

		m_nBufferSize = nSize;

		// Zero out newly reserved buffer
		if (m_nBufferSize > 0) {
			if (m_nBufferSize > nCurrentBufferSize)
				memset(&(m_pBuffer[nCurrentBufferSize]), 0, m_nBufferSize - nCurrentBufferSize);
		}

	}



	inline _T& operator[](int nIndex)
	{
		assert(nIndex >= 0);

		EnsureBufferCapacity(nIndex + 1);

		return m_pBuffer[nIndex];
	}

	inline const _T& operator[](int nIndex) const
	{
		assert(nIndex < m_nBufferSize);
		assert(nIndex >= 0);

		return m_pBuffer[nIndex];
	}

	inline _T* GetPtr(int nIndex = 0, int nLength = 1)
	{
		assert(nIndex >= 0);

		EnsureBufferCapacity(nIndex + nLength);

		return &(m_pBuffer[nIndex]);
	}

	inline const _T* GetPtr(int nIndex = 0, int nLength = 1) const
	{
		assert((nIndex + nLength) <= m_nBufferSize);
		assert(nIndex >= 0);

		return &(m_pBuffer[nIndex]);
	}

	inline _T* operator->() {
		assert(m_bLocked);
		return m_pBuffer;
	}

	inline const _T* operator->() const {
		assert(m_bLocked);
		return m_pBuffer;
	}

	inline int GetBufferCapacity() const {
		return m_nBufferCap;
	}

	inline int GetBufferSize() const {
		return m_nBufferSize;
	}

	inline void Sort(int (*pFnCompare )(const void *, const void *), glibBufferSortingAlgorithm eAlg = glibBufferSortingAlgorithm_Quick ) {

		switch(eAlg) {
			case meBufferSortingAlgorithm_Quick:
				qsort(m_pBuffer, m_nBufferSize, sizeof(_T), pFnCompare);
				break;

		}
	}

	void operator=(const glibMemBuffer<_T>& oOther)
	{
		// Only copy unlocked buffers
		assert(!oOther.m_bLocked);		
		assert(oOther.m_nBufferCap != 0);

		// If buffer is too small to hold 
		if ((m_pBuffer != NULL) && (m_nBufferCap < oOther.m_nBufferCap))
		{
			delete[] m_pBuffer;

			m_nBufferCap = 0;
			m_nBufferSize = 0;

			m_pBuffer = NULL;

		}

		// make new buffer in order to hold the right side buffer's content.

		if (m_pBuffer == NULL)
			m_pBuffer = new _T[oOther.m_nBufferCap];

		// Copy content of right side buffer ....
		memcpy(m_pBuffer, oOther.m_pBuffer, oOther.m_nBufferCap * sizeof(_T));

		// Copy all other parameters ... 
		m_nBufferCap = oOther.m_nBufferCap;
		m_nBufferSize = oOther.m_nBufferSize;
		m_bLocked = oOther.m_bLocked;

	}

private:

	int ComputeBlockCapacity(int nCap) {

		return ((nCap / nGrowstep) + 1) * nGrowstep;

	}

	void EnsureBufferCapacity(int nSize)
	{	
		if (m_bLocked)
		{
			assert(nSize <= m_nBufferCap);
			assert(m_pBuffer != NULL);
		}

		_T* pTempPtr = NULL;

		int nNewSize = 0;

		if (nSize > m_nBufferCap) {

			nNewSize = ComputeBlockCapacity(nSize);

			pTempPtr = m_pBuffer;

			m_pBuffer = new _T[nNewSize];

			assert(m_pBuffer != NULL);

			m_nBufferCap = nNewSize;
		}

		if (pTempPtr != NULL) {

			if (m_nBufferSize > 0) {

				memcpy(m_pBuffer, pTempPtr, m_nBufferSize * sizeof(_T));

			}

			delete[] pTempPtr;

		}

	}

	_T*		m_pBuffer;
	int		m_nBufferCap;
	int		m_nBufferSize;
	bool	m_bLocked;

};


#endif //_MISCBUFFER_H_HEADER_INCLUDED_

