

#include "stdafx.h"

string MiscStringEncoding::UnicodeToAscii(const ucString& oUCString, bool& bContainsUCCharacters)
{
	string oResult;
	const short* szInString = oUCString.c_str();

	bContainsUCCharacters = false;

	for (int nCount = 0; szInString[nCount] != 0; nCount++ )
	{
		// The unicode string must not have any non ascii characters .. otherwise it has to be translated
		// into the desired format.

		unsigned short usCurrent = (unsigned short)szInString[nCount];

		if ((usCurrent <= 0x00ff) && (usCurrent >= 0)) {
			oResult += (char)usCurrent;
		}
		else {
			oResult += '?';
			bContainsUCCharacters = true;
		}

	}
	return oResult;
}

ucString MiscStringEncoding::AsciiToUnicode(const string& oUCString)
{
	basic_string<short> oResult;
	const char* szInString = oUCString.c_str();

	for (int nCount = 0; szInString[nCount] != 0; nCount++ )
	{
		// The unicode string must not have any non ascii characters .. otherwise it has to be translated
		// into the desired format.

		unsigned char usCurrent = (unsigned char)szInString[nCount];

		oResult += (short)usCurrent;

	}
	return oResult;
}

//int MiscStringEncoding::UnicodeStringToBuffer(const ucString& oUCString, 
//									unsigned char* szBuffer,
//									int nBufferCapacity, 
//									bool bFirstChars )
//{
//
//}

string MiscStringEncoding::UTF8ToAscii(const string& oSourceString, char cUnmappableChar, bool& bErr)
{
	string oResult;
	bErr = false;

	int nSize = (int)oSourceString.size();

	int nCatchState = 0;
	unsigned short cCatchCharacter = 0;

	for (int nCount = 0; nCount < nSize; nCount++) {

		unsigned char cCharacter__ = oSourceString.c_str()[nCount];
		unsigned short cCharacter = (unsigned short)cCharacter__;

		//oResult

		switch(nCatchState) {
		
			case 0:

				// Ascii character
				if (cCharacter <= 0x7f) {				// 0000 0000 0000 007F   0xxxxxxx
					oResult += (unsigned char)cCharacter;
				}
				else if ((cCharacter & 0xe0) == 0xc0) {	// 0000 0080 0000 07FF   110xxxxx 10xxxxxx

					cCatchCharacter = ((unsigned short)(cCharacter & 0x3f)) << 6;
					nCatchState = 1;
				}
				else if ((cCharacter & 0xf0) == 0xe0) {	// 0000 0800 0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx
					cCatchCharacter = ((unsigned short)(cCharacter & 0x3f)) << 12;
					nCatchState = 2;
				}
				else {
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				break;
			case 1:
				if ((cCharacter & 0xc0) != 0x80) {		
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				else {	// 0000 0080 0000 07FF   110xxxxx 10xxxxxx

					cCatchCharacter |= ((unsigned short)(cCharacter )) & 0x003f ;

					if (cCatchCharacter > 0x00ff) {
						oResult += cUnmappableChar;
					}
					else {
						oResult += (char)cCatchCharacter;
					}
					
					nCatchState = 0;
				}
				break;
			case 2:
				if ((cCharacter & 0xc0) != 0x80) {		
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				else {	// 0000 0800 0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx

					cCatchCharacter |= (((unsigned short)(cCharacter )) & 0x003f) << 6 ;
					nCatchState = 3;
				}
				break;
			case 3:
				if ((cCharacter & 0xc0) != 0x80) {		
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				else {	// 0000 0800 0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx

					cCatchCharacter |= (((unsigned short)(cCharacter )) & 0x003f)  ;

					if (cCatchCharacter > 0x00ff) {
						oResult += cUnmappableChar;
					}
					else {
						oResult += (char)cCatchCharacter;
					}
					nCatchState = 0;
				}
				break;
		}

	}

	if (nCatchState != 0) {
		bErr = true;
		//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence. End of String within multi character code."))
	}


	return oResult;
}

ucString MiscStringEncoding::UTF8ToUnicode(const string& oSourceString, bool& bErr)
{
	ucString oResult;
	bErr = false;

	int nSize = (int)oSourceString.size();

	int nCatchState = 0;
	unsigned short cCatchCharacter = 0;

	for (int nCount = 0; nCount < nSize; nCount++) {

		unsigned char cCharacter__ = oSourceString.c_str()[nCount];
		unsigned short cCharacter = (unsigned short)cCharacter__;

		//oResult

		switch(nCatchState) {
		
			case 0:

				// Ascii character
				if (cCharacter <= 0x7f) {				// 0000 0000 0000 007F   0xxxxxxx
					oResult += (unsigned char)cCharacter;
				}
				else if ((cCharacter & 0xe0) == 0xc0) {	// 0000 0080 0000 07FF   110xxxxx 10xxxxxx

					cCatchCharacter = ((unsigned short)(cCharacter & 0x3f)) << 6;
					nCatchState = 1;
				}
				else if ((cCharacter & 0xf0) == 0xe0) {	// 0000 0800 0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx
					cCatchCharacter = ((unsigned short)(cCharacter & 0x3f)) << 12;
					nCatchState = 2;
				}
				else {
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				break;
			case 1:
				if ((cCharacter & 0xc0) != 0x80) {		
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				else {	// 0000 0080 0000 07FF   110xxxxx 10xxxxxx

					cCatchCharacter |= ((unsigned short)(cCharacter )) & 0x003f ;

					oResult += (short)cCatchCharacter;

					
					nCatchState = 0;
				}
				break;
			case 2:
				if ((cCharacter & 0xc0) != 0x80) {		
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				else {	// 0000 0800 0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx

					cCatchCharacter |= (((unsigned short)(cCharacter )) & 0x003f) << 6 ;
					nCatchState = 3;
				}
				break;
			case 3:
				if ((cCharacter & 0xc0) != 0x80) {		
					//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence."))
					bErr = true;
					return oResult;
				}
				else {	// 0000 0800 0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx

					cCatchCharacter |= (((unsigned short)(cCharacter )) & 0x003f)  ;

					oResult += (short)cCatchCharacter;

					nCatchState = 0;
				}
				break;
		}

	}

	if (nCatchState != 0) {
		bErr = true;
		//GLIB_THROW_CPP(glibException("glibStringUtil::AsciiToUC() : Malformed UTF8 character sequence. End of String within multi character code."))
	}


	return oResult;
}

