
#include "stdafx.h"

bool	MiscStringKeyEncoding::VerifyStringHash(const string& szInput, const string& oHash)
{
	unsigned char pInputHashBuffer[8];
	unsigned char pStringHashBuffer[8];

	MiscStringNumeric::HexStringToBinary(oHash, pInputHashBuffer, 8);

	string oComputedHashString = ComputeStringHash(szInput);

	MiscStringNumeric::HexStringToBinary(oComputedHashString, pStringHashBuffer, 8);

	for (int nCount = 0; nCount < 8; nCount++)
	{
		if (pInputHashBuffer[nCount] != pStringHashBuffer[nCount])
			return false;
	}

	return true;
}

string	MiscStringKeyEncoding::ComputeStringHash(const string& szInput)
{
	int nCount;
	const char* pCString = szInput.c_str();
	unsigned char oUserHashKey64[8];

	unsigned char ucLastChar = '-';

	// Reset hash code
	for (nCount = 0; nCount < 8; nCount++) {
		oUserHashKey64[nCount] = 0;
	}

	// Compute initial value for first rotator ...
	unsigned char cuRot1 = 0;
	for (nCount = 0; pCString[nCount] != 0; nCount++)
	{
		// make sure that swapped characters produce a different hash code
		cuRot1 += pCString[nCount] ^ ROL(pCString[nCount], nCount) ^ ROR(cuRot1, nCount);
		ucLastChar = pCString[nCount];
	}

	
	unsigned char cuRot2 = 0xa6;
	unsigned char ucOld = 0;

	bool bStringExceed = false;

	// compute hash code for specified user name
	for (nCount = 0; (pCString[nCount] != 0) || (nCount < 8); nCount++)
	{
		
		if (pCString[nCount] == 0)
			bStringExceed = true;

		cuRot1 ^= (!bStringExceed)?((unsigned char)(pCString[nCount])):(ucLastChar);
		cuRot2 ^= ROL(cuRot2, nCount >> 1) ^ ROR(cuRot1, nCount) ^ ucOld;
		cuRot1 = ROL(cuRot1, nCount << 2) + 
				(ROL(ROR(ucOld, nCount % 11) ^ 0xaa, nCount % 17) ^ ROR(cuRot2, ucOld)) + 
				nCount;

		oUserHashKey64[nCount % 8] ^= (unsigned char)(((!bStringExceed)?((unsigned char)(pCString[nCount])):(ucLastChar)) + nCount) ^ (cuRot1 + cuRot2);

		ucOld = oUserHashKey64[nCount % 8];

	}

	string oResult = MiscStringNumeric::BinaryToHexString(oUserHashKey64, 8);

	return oResult;
}

unsigned char MiscStringKeyEncoding::ROR(unsigned char cCharacter, int nRor)
{
	unsigned char ucRet = cCharacter;
	int nRorTrunc = nRor & 7;

	for (int ncount = 0; ncount < nRor; ncount++) {

		ucRet = (ucRet >> 1) | (ucRet << 7);

	}
	return ucRet;
}

unsigned char MiscStringKeyEncoding::ROL(unsigned char cCharacter, int nRol)
{
	unsigned char ucRet = cCharacter;
	int nRorTrunc = nRol & 7;

	for (int ncount = 0; ncount < nRol; ncount++) {

		ucRet = (ucRet << 1) | (ucRet >> 7);

	}
	return ucRet;
}

string	MiscStringKeyEncoding::BuildUniqueId(const string& oInput)
{
	vector<string> oTokens;

	MiscStringTokenizing::Tokenize(oInput, ":", oTokens);

	if (oTokens.size() != 2)
	{
		return "invalid";
	}

	string oSequence = oTokens[1];

	unsigned char szBuffer[8];

	memset(szBuffer, 0, 8 * sizeof(unsigned char));
	
	for (int ncount = 0; oSequence.c_str()[ncount] != 0; ncount++)
	{
		char cChar = oSequence.c_str()[ncount];

		int nFig = 0;

		if ((cChar >= '0') && (cChar <= '9'))
		{
			nFig = (int)(cChar - '0');
		}
		else if ((cChar >= 'a') && (cChar <= 'f'))
		{
			nFig = (int)(cChar - 'a' ) + 10;
		}
		else if ((cChar >= 'A') && (cChar <= 'F'))
		{
			nFig = (int)(cChar - 'A' ) + 10;
		}

		szBuffer[ncount & 7] += nFig;
		szBuffer[ncount & 7] = szBuffer[ncount & 7] & 0xf;

	}

	string oResult = oTokens[0] + ":";

	for (int nCount = 0; nCount < 8; nCount++)
	{
		char szBuffer_[2];

		szBuffer_[1] = 0;

		if ((szBuffer[nCount] >= 0) && (szBuffer[nCount] <= 9))
		{
			szBuffer_[0] = (char)(szBuffer[nCount] + '0');
		}
		else if ((szBuffer[nCount] >= 10) && (szBuffer[nCount] <= 15))
		{
			szBuffer_[0] = (char)(szBuffer[nCount] - 10 + 'a');
		} 
		else {
			szBuffer_[0] = 'x';
		}

		oResult += szBuffer_;

	}

	return oResult;
}

string	MiscStringKeyEncoding::BlockCharacters(const string& oInput, int nCharacters, int nIndent)
{
	string oResult;
	const char* szInput = oInput.c_str();

	int nCharCount = 0;
	for (int nCount = 0; szInput[nCount] != 0; nCount++)
	{

		char cCurrent = szInput[nCount];
		if ((cCurrent == '\r') ||
			(cCurrent == '\n') ||
			(cCurrent == '\t') ||
			(cCurrent == ' '))
			continue;

		if (nCharCount == 0)
		{
			for (int nIndentCount = 0; nIndentCount < nIndent; nIndentCount++)
				oResult += '\t';
		}

		oResult += cCurrent;

		if (((nCharCount % nCharacters) == (nCharacters - 1)) )
		{
			oResult += '\n';
			for (int nIndentCount = 0; nIndentCount < nIndent; nIndentCount++)
				oResult += '\t';
		}
		nCharCount++;
	}
	return oResult;
}

string	MiscStringKeyEncoding::DecryptString(const string& oInput, const string& oHash)
{
	int nCount;
	unsigned char pHashBuffer[8];
	unsigned char pStringHashBuffer[8];
	unsigned char pComputedStringHashBuffer[8];
	MiscStringNumeric::HexStringToBinary(oHash, pHashBuffer, 8);
	MiscStringNumeric::HexStringToBinary(oInput, pStringHashBuffer, 8);

	if (oInput.length() < 16)
		return string("TooShort");

	// truncate input string , because contained data starts from (non space) character 16 .... 
	string oRemainingInput = &(MiscStringUtil::TrimString(oInput).c_str()[16]);
	
	// compute binary buffer
	unsigned char szBuffer[16384];
	int nBytes = MiscStringNumeric::HexStringToBinary(oRemainingInput, szBuffer, 16384);
	
	// Terminating 0 is NOT encrypted, but contained.
	for (nCount = 0; nCount < nBytes - 1; nCount++)
	{
		szBuffer[nCount] ^= pHashBuffer[nCount % 8];
		szBuffer[nCount] ^= pStringHashBuffer[nCount % 8];
	}

	string oResult = (char*)szBuffer;
	string oStringHash = ComputeStringHash(oResult);
	MiscStringNumeric::HexStringToBinary(oStringHash, pComputedStringHashBuffer, 8);

	for (nCount = 0; nCount < 8; nCount++)
	{
		if (pComputedStringHashBuffer[nCount] != pStringHashBuffer[nCount])
			return string("NoMatch");
	}	
	return oResult;
}

string	MiscStringKeyEncoding::EncryptString(const string& oInput, const string& oHash)
{
	unsigned char pHashBuffer[8];
	unsigned char pStringHashBuffer[8];
	unsigned char szBuffer[16384];
	MiscStringNumeric::HexStringToBinary(oHash, pHashBuffer, 8);

	string oStringHash = ComputeStringHash(oInput);
	MiscStringNumeric::HexStringToBinary(oStringHash, pStringHashBuffer, 8);

	int nBytesInBuffer = (int)(oInput.length()) + 1;
	if (nBytesInBuffer > 16384)
		return string("TooLong");

	// copy raw string data to temporary buffer on stack
	memcpy(szBuffer, oInput.c_str(), oInput.length() + 1);
	
	// simple XOR encryption
	for (int nCount = 0; szBuffer[nCount] != 0; nCount++)
	{
		szBuffer[nCount] ^= pHashBuffer[nCount % 8];
		szBuffer[nCount] ^= pStringHashBuffer[nCount % 8];
	}

	string oHexString = MiscStringNumeric::BinaryToHexString(szBuffer, 
										  nBytesInBuffer);

	return oStringHash + oHexString;
	// Format is: yyyyyyyyyyyyyyyyxxxxxxxxxxxx.....x
	// where y is the checksum and x is the actual string
}

string	MiscStringKeyEncoding::EssentializeString(const string& oInput)
{
	string oResult;

	const char* szInp = oInput.c_str();

	for (int nCount = 0; szInp[nCount] != 0; nCount++ )
	{
		char ucCharacter = szInp[nCount];

		// make uppercase
		if ((ucCharacter >= 'a') && (ucCharacter <= 'z'))
		{
			oResult += (ucCharacter - 'a' + 'A');
		}
		else if ((ucCharacter >= 'A') && (ucCharacter <= 'Z'))
		{
			oResult += ucCharacter;
		}
		else if ((ucCharacter >= '0') && (ucCharacter <= '9'))
		{
			oResult += ucCharacter;
		}
		else if ((ucCharacter == '') || (ucCharacter == '') || (ucCharacter == '') || (ucCharacter == ''))
		{
			oResult += ucCharacter;
		}
		else if (ucCharacter == '')
		{
			oResult += '';
		}
		else if (ucCharacter == '')
		{
			oResult += '';
		}
		else if (ucCharacter == '')
		{
			oResult += '';
		}
	}

	return oResult;
}

string	MiscStringKeyEncoding::ComputeHashConditionized(const string& oInput)
{
	return ComputeStringHash(EssentializeString(oInput));
}