
#include "stdafx.h"

float MiscStringNumeric::StringToFloat(const string& oInput, bool& bUnexpectedChar, bool bScientific)
{
	const char* szTarget = oInput.c_str();

	// Lots of flags
	bool bSignum = false;
	float fInteger = 0.0f;
	float fFractal = 0.1f;
	int nExponent = 0;
	bool bExponentSignum = false;
	float fExponent = 0.0f;
	
	bool bPreMantissa = true;
	bool bInteger = false;
	bool bFractal = false;
	bool bPreExponent = false;
	bool bExponent = false;
	
	bUnexpectedChar = false;

	for (int nCount = 0; szTarget[nCount] != 0; nCount++  )
	{
		char cCharacter = szTarget[nCount];
		
		// Ignore whitespace (we are that tolerant)
		if ((cCharacter == ' ') || 
			(cCharacter == '\t'))
			continue;


		//---------------------------------------------------------------------------------------------
		//  Get First valid character
		//---------------------------------------------------------------------------------------------
		if (bPreMantissa) {
		
			// Only '-', '.' or a figure are allowed as the first chracter ....
			if (cCharacter == '-') {
				bSignum = true;
				bPreMantissa = false;
				bInteger = true;
				continue;
			}
			else if ((cCharacter == ',') || (cCharacter == '.')) {
				bPreMantissa = false;
				bInteger = true;
			}
			else if (cCharacter >= '0' && cCharacter <= '9')
			{
				bPreMantissa = false;
				bInteger = true;				
			}
			else {
				bUnexpectedChar = true;
			}
		
		}
		if (bInteger) {	// Integer part of the value
		
			if (cCharacter >= '0' && cCharacter <= '9')
			{
				fInteger = fInteger * 10.0f + (float)(cCharacter - '0');
			}		
			else if ((cCharacter == ',') || (cCharacter == '.')) {	// Decimal point ... changing to frature part
				bFractal = true;
				bInteger = false;
				continue;
			}
			else if ((cCharacter == 'e') || (cCharacter == 'E')) { // Exponent
				bPreExponent = true;
				bInteger = false;
				continue;
			}	
			else {
				bUnexpectedChar = true;
			}
		}
		
		// The fractual part of the value (after decimal dot)
		if (bFractal) {
		
			if (cCharacter >= '0' && cCharacter <= '9')
			{
				fInteger += fFractal * (float)(cCharacter - '0');
				fFractal *= 0.1f;
			}	
			else if (bScientific && ((cCharacter == 'e') || (cCharacter == 'E'))) { // Exponent
				bPreExponent = true;
				bFractal = false;
				continue;
			}	
			else {
				bUnexpectedChar = true;
			}
						
		}				
		if (bPreExponent) { // Before parsing the exponent ... could have a '-'
		
			if (cCharacter == '-') {		// negative exponent
				bExponentSignum = true;
				bPreExponent = false;
				bExponent = true;
				continue;
			}	
			else if (cCharacter >= '0' && cCharacter <= '9')	// The exponent value starts
			{
				bPreExponent = false;
				bExponent = true;				
			}
			else {
				bUnexpectedChar = true;
			}
		}						
		if (bExponent) {
		
			if (cCharacter >= '0' && cCharacter <= '9') // Compute exponent
			{
		
				nExponent = nExponent * 10 + (unsigned int)(cCharacter - '0');
			}
			else {
				bUnexpectedChar = true;
			}
		
		}						
	}	
	
	// Assign signum to exponent
	if (bExponentSignum) {
		
		nExponent = -nExponent;
	
	}
	
	// Assign signum to value
	if (bSignum) {
		
		fInteger = -fInteger;
		
	}
	
	// If we got stuck in 'pre mantissa', we did not catch a valid starting character.
	if (bPreMantissa) {
		bUnexpectedChar = true;
	}

	if (nExponent != 0) {
		float fExp = exp(((float)nExponent) * 2.3025850929940456840179914546844f);
		fInteger = fExp * fInteger;
	}

	// Now add the specified exponent to 
	//AddToBiasedExponent(fInteger, nExponent);
	
	return fInteger;
}

double MiscStringNumeric::StringToDouble(const string& oInput, bool& bUnexpectedChar, bool bScientific)
{
	const char* szTarget = oInput.c_str();

	// Lots of flags
	bool bSignum = false;
	double fInteger = 0.0f;
	double fFractal = 0.1f;
	int nExponent = 0;
	bool bExponentSignum = false;
	double fExponent = 0.0f;
	
	bool bPreMantissa = true;
	bool bInteger = false;
	bool bFractal = false;
	bool bPreExponent = false;
	bool bExponent = false;
	
	bUnexpectedChar = false;

	for (int nCount = 0; szTarget[nCount] != 0; nCount++  )
	{
		char cCharacter = szTarget[nCount];
		
		// Ignore whitespace (we are that tolerant)
		if ((cCharacter == ' ') || 
			(cCharacter == '\t'))
			continue;


		//---------------------------------------------------------------------------------------------
		//  Get First valid character
		//---------------------------------------------------------------------------------------------
		if (bPreMantissa) {
		
			// Only '-', '.' or a figure are allowed as the first chracter ....
			if (cCharacter == '-') {
				bSignum = true;
				bPreMantissa = false;
				bInteger = true;
				continue;
			}
			else if ((cCharacter == ',') || (cCharacter == '.')) {
				bPreMantissa = false;
				bInteger = true;
			}
			else if (cCharacter >= '0' && cCharacter <= '9')
			{
				bPreMantissa = false;
				bInteger = true;				
			}
			else {
				bUnexpectedChar = true;
			}
		
		}
		if (bInteger) {	// Integer part of the value
		
			if (cCharacter >= '0' && cCharacter <= '9')
			{
				fInteger = fInteger * 10.0 + (double)(cCharacter - '0');
			}		
			else if ((cCharacter == ',') || (cCharacter == '.')) {	// Decimal point ... changing to frature part
				bFractal = true;
				bInteger = false;
				continue;
			}
			else if ((cCharacter == 'e') || (cCharacter == 'E')) { // Exponent
				bPreExponent = true;
				bInteger = false;
				continue;
			}	
			else {
				bUnexpectedChar = true;
			}
		}
		
		// The fractual part of the value (after decimal dot)
		if (bFractal) {
		
			if (cCharacter >= '0' && cCharacter <= '9')
			{
				fInteger += fFractal * (double)(cCharacter - '0');
				fFractal *= 0.1;
			}	
			else if (bScientific && ((cCharacter == 'e') || (cCharacter == 'E'))) { // Exponent
				bPreExponent = true;
				bFractal = false;
				continue;
			}	
			else {
				bUnexpectedChar = true;
			}
						
		}				
		if (bPreExponent) { // Before parsing the exponent ... could have a '-'
		
			if (cCharacter == '-') {		// negative exponent
				bExponentSignum = true;
				bPreExponent = false;
				bExponent = true;
				continue;
			}	
			else if (cCharacter >= '0' && cCharacter <= '9')	// The exponent value starts
			{
				bPreExponent = false;
				bExponent = true;				
			}
			else {
				bUnexpectedChar = true;
			}
		}						
		if (bExponent) {
		
			if (cCharacter >= '0' && cCharacter <= '9') // Compute exponent
			{
		
				nExponent = nExponent * 10 + (unsigned int)(cCharacter - '0');
			}
			else {
				bUnexpectedChar = true;
			}
		
		}						
	}	
	
	// Assign signum to exponent
	if (bExponentSignum) {
		
		nExponent = -nExponent;
	
	}
	
	// Assign signum to value
	if (bSignum) {
		
		fInteger = -fInteger;
		
	}
	
	// If we got stuck in 'pre mantissa', we did not catch a valid starting character.
	if (bPreMantissa) {
		bUnexpectedChar = true;
	}

	if (nExponent != 0) {
		float fExp = exp(((double)nExponent) * 2.3025850929940456840179914546844);
		fInteger = fExp * fInteger;
	}

	

	// Now add the specified exponent to 
	//AddToBiasedExponent(fInteger, nExponent);
	
	return fInteger;
}

unsigned int MiscStringNumeric::WebColorToXBGR(const string& oWebClr)
{
	unsigned int unReturnVal = 0;
	const char* szWebClr = oWebClr.c_str();

	int nFigure = 0;
	int nState = 0;
	unsigned short cRed = 0;
	unsigned short cGreen = 0;
	unsigned short cBlue = 0; 

	for (int ncount = 0; szWebClr[ncount] != 0; ncount++)
	{
		char cCurrent = szWebClr[ncount];

		int nVal = 0;

		if ((cCurrent >= '0') && (cCurrent <= '9'))
		{
			
			nVal = cCurrent - '0';
		}
		else if ((cCurrent >= 'a') && (cCurrent <= 'f'))
		{
			
			nVal = cCurrent - 'a' + 10;
		}
		else if ((cCurrent >= 'A') && (cCurrent <= 'F'))
		{
			
			nVal = cCurrent - 'A' + 10;
		}
		else 
			continue;

		// inc state after 2 figures (8 bit)



		switch(nState)
		{
		case 0: // r
			cRed = cRed * 16 + nVal;
			break;
		case 1:	// g
			cGreen = cGreen * 16 + nVal;
			break;
		case 2:	// b
			cBlue = cBlue * 16 + nVal;
			break;
		}

		nFigure++;
		if (nFigure > 1)
		{
			nFigure = 0;
			nState++;
			
		}

		// stop after 2 values
		if (nState > 2)
			break;

	}

	unReturnVal = ((unsigned int)cRed);
	unReturnVal |= ((unsigned int)cGreen) << 8;
	unReturnVal |= ((unsigned int)cBlue)<< 16;

	return unReturnVal;
}
string MiscStringNumeric::XBGRToWebColor(unsigned int unXRGB)
{
	char szBuffer[128];

	sprintf(szBuffer, "#%02x%02x%02x",  unXRGB & 0x000000ff,
										(unXRGB & 0x0000ff00) >> 8,
										(unXRGB & 0x00ff0000) >> 16);

	return string(szBuffer);
}

string MiscStringNumeric::XBGRToHexColor(unsigned int unXRGB)
{
	char szBuffer[128];

	sprintf(szBuffer, "%02x%02x%02x",  unXRGB & 0x000000ff,
										(unXRGB & 0x0000ff00) >> 8,
										(unXRGB & 0x00ff0000) >> 16);

	return string(szBuffer);
}

string	MiscStringNumeric::BinaryToHexString(const unsigned char* pBinBlock, int nSize)
{
	string oResult;

	if (nSize == 0)
		return "X";

	for (int nCount = 0; nCount < nSize; nCount++)
	{
		unsigned char ucCharacter = pBinBlock[nCount];
		
		unsigned char ucUpperNibble = (ucCharacter >> 4) & 0x0f;
		unsigned char ucLowerNibble = ucCharacter  & 0x0f;

		char cUpperHexchar = '0';
		char cLowerHexchar = '0';

		if ((ucUpperNibble >= 0x00) && (ucUpperNibble <= 0x09))
		{
			cUpperHexchar = '0' + (char)ucUpperNibble;
		}
		else 
		{
			cUpperHexchar = 'A' + (char)(ucUpperNibble - 0x0a);
		}

		if ((ucLowerNibble >= 0x00) && (ucLowerNibble <= 0x09))
		{
			cLowerHexchar = '0' + (char)ucLowerNibble;
		}
		else 
		{
			cLowerHexchar = 'A' + (char)(ucLowerNibble - 0x0a);
		}

		oResult += cUpperHexchar;
		oResult += cLowerHexchar;

	}



	return oResult;

}



int MiscStringNumeric::HexStringToBinary(const string& oHexString, unsigned char* pBinBlock, int nCapacity)
{
	if (oHexString == "X")
	{
		return 0;
	}

	const char* szHexString = oHexString.c_str();

	int nTargetPosition = 0;
	int nState = 0;

	unsigned char ucBinValue = 0;

	for (int nCount = 0; szHexString[nCount] != 0; nCount++)
	{
		char ucCharacter = szHexString[nCount];
		unsigned char ucNibbleVal = 0;

		if (((ucCharacter > '9') || (ucCharacter < '0')) && 
			((ucCharacter > 'f') || (ucCharacter < 'a')) &&
			((ucCharacter > 'F') || (ucCharacter < 'A')))
			continue;

		if ((ucCharacter <= 'f') && (ucCharacter >= 'a'))
		{
			ucCharacter -= 'a';
			ucCharacter	+= 'A';
		}

		if ((ucCharacter <= '9') && (ucCharacter >= '0'))
		{
			ucNibbleVal = (unsigned char)ucCharacter - '0';
		}
		else 
		{
			ucNibbleVal = (unsigned char)ucCharacter - 'A' + 10;
		}

		switch(nState)
		{
		case 0:
			ucBinValue = ucNibbleVal << 4;
			nState = 1;
			break;
		case 1:
			ucBinValue |= ucNibbleVal;
			pBinBlock[nTargetPosition++] = ucBinValue;
			if (nTargetPosition >= nCapacity)
				return nTargetPosition;
			nState = 0;
			break;
		}
	}
	return nTargetPosition;
}
//int		MiscStringNumeric::HexStringToBinary(	const string& oHexString, 
//							unsigned char* pBinBlock, 
//							int nCapacity)
//{
//	if (oHexString == "X")
//	{
//		return 0;
//	}
//
//	const char* szHexString = oHexString.c_str();
//
//	int nTargetPosition = 0;
//	int nState = 0;
//
//	unsigned char ucBinValue = 0;
//
//	for (int nCount = 0; szHexString[nCount] != 0; nCount++)
//	{
//		char ucCharacter = szHexString[nCount];
//		unsigned char ucNibbleVal = 0;
//
//		if (((ucCharacter > '9') || (ucCharacter < '0')) && 
//			((ucCharacter > 'f') || (ucCharacter < 'a')) &&
//			((ucCharacter > 'F') || (ucCharacter < 'A')))
//			continue;
//
//		if ((ucCharacter <= 'f') && (ucCharacter >= 'a'))
//		{
//			ucCharacter -= 'a';
//			ucCharacter	+= 'A';
//		}
//
//		if ((ucCharacter <= '9') && (ucCharacter >= '0'))
//		{
//			ucNibbleVal = (unsigned char)ucCharacter - '0';
//		}
//		else 
//		{
//			ucNibbleVal = (unsigned char)ucCharacter - 'A' + 10;
//		}
//
//		switch(nState)
//		{
//		case 0:
//			ucBinValue = ucNibbleVal << 4;
//			nState = 1;
//			break;
//		case 1:
//			ucBinValue |= ucNibbleVal;
//			pBinBlock[nTargetPosition++] = ucBinValue;
//			if (nTargetPosition >= nCapacity)
//				return nTargetPosition;
//			nState = 0;
//			break;
//		}
//	}
//	return nTargetPosition;
//}

int MiscStringNumeric::StringToInt(const string& oTarget, bool& bUnexpectedChar)
{
	return StringToInt(oTarget.c_str(), bUnexpectedChar);
}

float MiscStringNumeric::StringToFloat(const char* szTarget, bool& bUnexpectedChar, bool bScientific)
{
	// Lots of flags
	bool bSignum = false;
	float fInteger = 0.0f;
	float fFractal = 0.1f;
	int nExponent = 0;
	bool bExponentSignum = false;
	float fExponent = 0.0f;
	
	bool bPreMantissa = true;
	bool bInteger = false;
	bool bFractal = false;
	bool bPreExponent = false;
	bool bExponent = false;
	
	bUnexpectedChar = false;

	for (int nCount = 0; szTarget[nCount] != 0; nCount++  )
	{
		char cCharacter = szTarget[nCount];
		
		// Ignore whitespace (we are that tolerant)
		if ((cCharacter == ' ') || 
			(cCharacter == '\t'))
			continue;

		//---------------------------------------------------------------------------------------------
		//  Get First valid character
		//---------------------------------------------------------------------------------------------
		if (bPreMantissa) {
		
			// Only '-', '.' or a figure are allowed as the first chracter ....
			if (cCharacter == '-') {
				bSignum = true;
				bPreMantissa = false;
				bInteger = true;
				continue;
			}
			else if (cCharacter == '.') {
				bPreMantissa = false;
				bInteger = true;
			}
			else if (cCharacter >= '0' && cCharacter <= '9')
			{
				bPreMantissa = false;
				bInteger = true;				
			}
			else {
				bUnexpectedChar = true;
			}
		}
		if (bInteger) {	// Integer part of the value
		
			if (cCharacter >= '0' && cCharacter <= '9')
			{
				fInteger = fInteger * 10.0f + (float)(cCharacter - '0');
			}		
			else if (cCharacter == '.') {	// Decimal point ... changing to frature part
				bFractal = true;
				bInteger = false;
				continue;
			}
			else if ((cCharacter == 'e') || (cCharacter == 'E')) { // Exponent
				bPreExponent = true;
				bInteger = false;
				continue;
			}	
			else {
				bUnexpectedChar = true;
			}
		}
		// The fractual part of the value (after decimal dot)
		if (bFractal) {
		
			if (cCharacter >= '0' && cCharacter <= '9')
			{
				fInteger += fFractal * (float)(cCharacter - '0');
				fFractal *= 0.1f;
			}	
			else if (bScientific & ((cCharacter == 'e') || (cCharacter == 'E'))) { // Exponent
				bPreExponent = true;
				bFractal = false;
				continue;
			}	
			else {
				bUnexpectedChar = true;
			}		
		}				
		if (bPreExponent) { // Before parsing the exponent ... could have a '-'
		
			if (cCharacter == '-') {		// negative exponent
				bExponentSignum = true;
				bPreExponent = false;
				bExponent = true;
				continue;
			}	
			else if (cCharacter >= '0' && cCharacter <= '9')	// The exponent value starts
			{
				bPreExponent = false;
				bExponent = true;				
			}
			else {
				bUnexpectedChar = true;
			}
		}						
		if (bExponent) {
			if (cCharacter >= '0' && cCharacter <= '9') // Compute exponent
			{
				nExponent = nExponent * 10 + (unsigned int)(cCharacter - '0');
			}
			else {
				bUnexpectedChar = true;
			}
		}						
	}	
	
	// Assign signum to exponent
	if (bExponentSignum) {
		nExponent = -nExponent;
	}
	
	// Assign signum to value
	if (bSignum) {
		fInteger = -fInteger;
	}
	
	// If we got stuck in 'pre mantissa', we did not catch a valid starting character.
	if (bPreMantissa) {
		bUnexpectedChar = true;
	}

	if (nExponent != 0) {
		float fExp = exp(((float)nExponent) * 2.3025850929940456840179914546844f);
		fInteger = fExp * fInteger;
	}

	// Now add the specified exponent to 
	//AddToBiasedExponent(fInteger, nExponent);
	
	return fInteger;
}

int MiscStringNumeric::StringToInt(const char* szTarget, bool& bUnexpectedChar)
{
	int nValue = 0;
	
	// Lots of flags ......
	bool bVirginity = true;
	bool bAwaiting = false;
	bool bIsHex = false;
	bool bIsBinary = false;
	bool bIsOctal = false;
	bool bIsDecimal = false;
	bool bIsNegative = false;

	bUnexpectedChar = false;
	
	for (int nCount = 0; szTarget[nCount] != 0; nCount++  )
	{
	
		char cCharacter = szTarget[nCount];

		if ((cCharacter == ' ') || 
			(cCharacter == '\t'))
			continue;


		//------------------------------------------------------------------------------------------
		//  Is there a leading '0', then we await a switch to either octal, binary, hex or decimal.
		//------------------------------------------------------------------------------------------

		if (bAwaiting) {	

			if ((cCharacter <= '9') &&
				(cCharacter >= '0'))	// Decimal mode ....
			{
				bAwaiting = false;
				bIsDecimal = true;
			}
			else
			if (cCharacter == 'x') {	// hex mode
				bAwaiting = false;
				bIsHex = true;
				continue;
			}
			else
			if (cCharacter == 'o') {	// octal mode
				bAwaiting = false;
				bIsOctal = true;
				continue;
			}
			else
			if (cCharacter == 'b') {	// Binary mode
				bAwaiting = false;
				bIsBinary = true;
				continue;
			}
			else 
				bUnexpectedChar = true;

		}

		//---------------------------------------------------------------------------------------------
		//  If no figure was detected yet then we are still innocent (like a virgin)
		//---------------------------------------------------------------------------------------------

		if (bVirginity) {
			if (cCharacter == '-')  {
				
				bIsNegative = true;
			}
			else if (cCharacter == '0') {
				
				bVirginity = false; // Lost innocence
				bAwaiting = true;	// mode not sure yet .... 
			}
			else if ((cCharacter > '0') && (cCharacter <= '9') ) {
				
				bVirginity = false;	// Lost innocence
				bIsDecimal = true;	// determined to be decimal !
			}
			else {
				bUnexpectedChar = true;
			}
		}		

		//---------------------------------------------------------------------------------------------
		//  If a mode is selected, then we can go and parse, knowing which format to expect !
		//---------------------------------------------------------------------------------------------

		if(bIsDecimal) {		// Parsing decimal
			if (cCharacter >= '0' && cCharacter <= '9') {
				nValue = 10 * nValue + (int)(cCharacter - '0');
				continue;
			}
			else {
				// Report evil characters, because we are interested in well formed integers only
				bUnexpectedChar = true;
			}
		}
		else if(bIsHex) {		// Parsing hex

			if (cCharacter >= '0' && cCharacter <= '9') {
				nValue = 16 * nValue + (int)(cCharacter - '0');
				continue;			
			}
			else if (cCharacter >= 'a' && cCharacter <= 'f') {
				nValue = 16 * nValue + (int)(cCharacter - 'a') + 10;
				continue;						
			}		
			else if (cCharacter >= 'A' && cCharacter <= 'F') {
				nValue = 16 * nValue + (int)(cCharacter - 'A') + 10;
				continue;			
			}		
			else {
				// Report evil characters, because we are interested in well formed integers only
				bUnexpectedChar = true;
			}
		}
		else if(bIsBinary) {	// Parsing binary
			if (cCharacter == '0' || cCharacter == '1') {
			
				nValue = 2 * nValue + (int)(cCharacter - '0');
				continue;
			}
			else if (cCharacter == 't' || cCharacter == 'T') {
				nValue = 2 * nValue + 1;
				continue;				
			}		
			else if (cCharacter == 'f' && cCharacter == 'F') {
				nValue = 2 * nValue;
				continue;	
			}		
			else {
				// Report evil characters, because we are interested in well formed integers only
				bUnexpectedChar = true;
			}
		}
		else if (bIsOctal) {	// Parsing octal
			if (cCharacter >= '0' && cCharacter <= '7') {
				nValue = 8 * nValue + (int)(cCharacter - '0');
				continue;			
			}
			else {
				// Report evil characters, because we are interested in well formed integers only
				bUnexpectedChar = true;
			}
		}
	}

	// Negative sign
	if (bIsNegative) {
		return -nValue;
	}

	// Report malformed figures
	return nValue;
	
}
