
#include "stdafx.h"

string MiscStringParsing::m_oTagOpenSequence("$$%%$$%%__");
string MiscStringParsing::m_oTagCloseSequence("__%%$$%%$$");

bool MiscStringParsing::ParseOption(MiscStringParsing_OptionParserIF* pReceiver, const string& oOptionString)
{
	if (pReceiver == NULL)
		return false;

	vector<string> oOptionParts;
	MiscStringTokenizing::Tokenize(oOptionString, "=", oOptionParts);

	if (oOptionParts.size() == 0) {
		return false;
	}

	string oOptionKey = oOptionParts[0];

	if (oOptionParts.size() == 1)
	{
		return pReceiver->OnOption(oOptionKey);
		
	}

	string oOptionValue = oOptionParts[1];

	if (oOptionParts.size() >= 2)
	{
		vector<string> oOptionValues;
		MiscStringTokenizing::Tokenize(oOptionValue, ",", oOptionValues);

		if (oOptionValues.size() == 0)
			return true;

		if (oOptionValues.size() == 1) {
			
			return pReceiver->OnOption(oOptionKey, oOptionValues[0]);
		}

		if (oOptionValues.size() > 1) {
			return pReceiver->OnOption(oOptionKey, oOptionValue, oOptionValues);
		}

	}

	return false;
}

void MiscStringParsing::SetTagOpener(const string& oOpen)
{
	m_oTagOpenSequence = oOpen;
}

void MiscStringParsing::SetTagCloser(const string& oClose)
{
	m_oTagCloseSequence = oClose;
}

string MiscStringParsing::CreateTag(const string& oSection, int nPhraseIndex)
{
	string oResult;
	
	char szBuffer[128];

	sprintf(szBuffer, "%05d", nPhraseIndex);

	oResult += m_oTagOpenSequence;
	oResult += oSection;
	oResult += "_";
	oResult += szBuffer;
	oResult += m_oTagCloseSequence;
	

	return oResult;
}

int MiscStringParsing::GetTagStdLength()
{
	return  2 +								// Length of section id
			1 +								// Length of separator
			5 +								// Length of numeric id
			(int)m_oTagCloseSequence.length() +
			(int)m_oTagOpenSequence.length();
}

bool MiscStringParsing::AnalyseTag(const string& oTag, 
									string& oSection, 
									int& nPhraseIndex)
{
	unsigned int nPosOpen = (unsigned int)oTag.find(m_oTagOpenSequence);
	unsigned int nPosClose = (unsigned int)oTag.find(m_oTagCloseSequence);

	// length must match !
	if (oTag.length() != GetTagStdLength())
		return false;

	// Open tag must exist
	if (nPosOpen == string::npos)
		return false;

	// Close tag must exist
	if (nPosClose == string::npos)
		return false;

	// Open tag must be first characters
	if (nPosOpen != 0)
		return false;

	// closing tag must be last characters
	if (nPosClose !=	2 +								// Length of section id
						1 +								// Length of separator
						5 +								// Length of numeric id
						m_oTagOpenSequence.length())
		return false;	


	// center underscore must match
	if(oTag.c_str()[2 + m_oTagOpenSequence.length()] != '_')
		return false;

	// Now analyze sequence

	oSection = "";
	oSection += oTag.c_str()[m_oTagOpenSequence.length() + 0];
	oSection += oTag.c_str()[m_oTagOpenSequence.length() + 1];

	nPhraseIndex = 0;
	for (int nCount = 0; nCount < 5; nCount++) {

		char cCharacter = oTag.c_str()[m_oTagOpenSequence.length() + 3 + nCount];

		if ((cCharacter > '9') || (cCharacter < '0'))
			return false;

		nPhraseIndex = 10 * nPhraseIndex + cCharacter - '0';

	}


	return true;
}


map<string, string> MiscStringParsing::AnalyzeFeatureTag(const string& oInput)
{
	// first tokenize by |

	map<string, string> oResult;

	string oTrimmedInput = MiscStringUtil::TrimString(oInput);
	vector<string> oTokens;

	MiscStringTokenizing::Tokenize(oInput, "|", oTokens);

	for (vector<string>::iterator it_tok = oTokens.begin();
		 it_tok != oTokens.end();
		 it_tok++)
	{
		string CurrentFeature = MiscStringUtil::TrimString(*it_tok);
		vector<string> oFeatureTokens;

		MiscStringTokenizing::Tokenize(CurrentFeature, "=", oFeatureTokens);

		switch(oFeatureTokens.size()) {

		case 1:
			oResult[oFeatureTokens[0]] = string("-");
			break;
		case 2:
			oResult[oFeatureTokens[0]] = oFeatureTokens[1];
			break;
		}

	}

	return oResult;
}

string MiscStringParsing::CreateFeatureTag(const map<string, string>& oFeatures)
{
	string oResult;
	map<string, string>::const_iterator it = oFeatures.begin();
	int nCount = 0;
	

	for (;
		 it != oFeatures.end();
		 it++, nCount++)
	{
		if (nCount > 0)
		{
			oResult += "|";
		}

		if ((it->second.length() < 1) || 
			(it->second == "-"))
		{
			oResult += it->first;
		}
		else {

			oResult += it->first;
			oResult += "=";
			oResult += it->second;

		}
	}

	return oResult;
}