


template<class _CharacterType>
class MiscStringQuoteTokenizer {

public:
	
	MiscStringQuoteTokenizer()
	{
		Reset();
	}
	MiscStringQuoteTokenizer(const _CharacterType* szString)
	{
		Set(szString);
	}
	MiscStringQuoteTokenizer(const basic_string<_CharacterType>& oString)
	{
		Set(oString);
	}

	void Set(const _CharacterType* szString)
	{
		Reset();
		basic_string<_CharacterType> oString = szString;
		Set(oString);
	}
	void Set(const basic_string<_CharacterType>& oString)
	{
		Reset();
		m_oBasicString = oString;
	}

	void Reset()
	{
		ResetPositions();
	}

	bool NextToken(const _CharacterType* szDelimiters, const _CharacterType* szQuotes)
	{
		

		if (m_nCurrentPosition_From < 0) {
			m_nCurrentPosition_From = 0;
			m_nCurrentPosition_To = 0;
		}

		m_nCurrentPosition_From = m_nCurrentPosition_To;

		const char* szOriginalString = m_oBasicString.c_str();
		bool bLastEsc = false;
		bool bLastQuot = false;
		bool bEmptyQuoteSense0 = false;
		bool bEmptyQuoteSense01 = false;
		bool bEmptyQuoteSense1 = false;

		while (szOriginalString[m_nCurrentPosition_To] != ((_CharacterType)0))
		{
			bLastQuot = m_bIsInQuote;

			bEmptyQuoteSense01 = bEmptyQuoteSense0;
			bEmptyQuoteSense0 = false;

			// Do Quote switch, but ignore escaped quotes: \" or \' 
			if (!m_bEscape) {
				if (IsDelimiter(szOriginalString[m_nCurrentPosition_To], szQuotes)) {

					if (!m_bIsInQuote) {
						m_bIsInQuote = true;
						bEmptyQuoteSense0 = true;
					}
					else {
						m_bIsInQuote = false;
						bEmptyQuoteSense1 = true;
					}

				}
			}

			bLastEsc = m_bEscape; // remember escape state of last character
			m_bEscape = false;
			if (szOriginalString[m_nCurrentPosition_To] == ((_CharacterType)'\\')) {
				m_bEscape = true;
			}

			// handle ""
			if ((bEmptyQuoteSense01) && (bEmptyQuoteSense1))
			{
				m_nCurrentPosition_To++;
				m_oResult = "";
				return true;
			}
			bEmptyQuoteSense1 = false;

			if ((!m_bIsInQuote) && 
				IsDelimiter(szOriginalString[m_nCurrentPosition_To], szDelimiters, szQuotes))
			{
				if (m_nCurrentPosition_To > m_nCurrentPosition_From) {
					m_oResult = m_oBasicString.substr(m_nCurrentPosition_From,
													  m_nCurrentPosition_To - m_nCurrentPosition_From);


					// was last quote a closing quote ??
					if ((bLastQuot) && (szOriginalString[m_nCurrentPosition_To] != 0))
					{
						// Otherwise we will get into trouble with quote state, due quote it 
						// parsed at the start of next token again.
						m_nCurrentPosition_To++;
					}

					return true;
				}
			}

			m_nCurrentPosition_To++;
			if (IsDelimiter(szOriginalString[m_nCurrentPosition_From], szDelimiters, szQuotes)) {

				// To is in first character after quote ... so fix from to it !
				if (m_bIsInQuote && !(bLastQuot)) {
					m_nCurrentPosition_From = m_nCurrentPosition_To;
				}
				else if (!m_bIsInQuote)
				{
				m_nCurrentPosition_From++;
			}
				
			}
		}

		if (m_nCurrentPosition_To == m_nCurrentPosition_From) {
			m_oResult = "";
			return false;
		}

		m_oResult = m_oBasicString.substr(m_nCurrentPosition_From,
										  m_nCurrentPosition_To - m_nCurrentPosition_From);

		return true;

	}
	bool NextToken(const basic_string<_CharacterType> oDelimiters,
				   const basic_string<_CharacterType> oQuotes)
	{
		bool bSucc = NextToken(oDelimiters.c_str(), oQuotes.c_str());
	}

	const basic_string<_CharacterType>& GetResult() const {

		return m_oResult;
		
	}

	const _CharacterType* GetStrResult() const {

		return m_oResult.c_str();
		
	}


private:

	void ResetPositions() {
		m_nCurrentPosition_From = -1;
		m_nCurrentPosition_To = -1;
		m_bIsInQuote = false;
		m_bEscape = false;
	}

	bool IsDelimiter(const _CharacterType szCharacter, const _CharacterType* szDelimiters) {

		for (int nCount = 0; szDelimiters[nCount] != (_CharacterType)0; nCount++) {

			if (szCharacter == szDelimiters[nCount]) 
				return true;

		}

		return false;
	}

	bool IsDelimiter(const _CharacterType szCharacter, 
					 const _CharacterType* szDelimiters,
					 const _CharacterType* szQuotes) 
	{

		int nCount;
		for (nCount = 0; szDelimiters[nCount] != (_CharacterType)0; nCount++) {

			if (szCharacter == szDelimiters[nCount]) 
				return true;

		}

		for (nCount = 0; szDelimiters[nCount] != (_CharacterType)0; nCount++) {

			if (szCharacter == szQuotes[nCount]) 
				return true;

		}

		return false;
	}

	basic_string<_CharacterType>	m_oBasicString;
	basic_string<_CharacterType>	m_oResult;

	int								m_nCurrentPosition_From;
	int								m_nCurrentPosition_To;
	bool							m_bIsInQuote;
	bool							m_bEscape;

};
